package de.ba_leipzig.discoinv;

import java.io.*;
import java.util.*;

public class Config {
	private static final String DB_PATH = "dbPath";
	private static final String DB_MANAGER = "dbManager";
	private static File file = new File("discoinv.conf");	
	private static Properties properties = new Properties();
	
	static {
		synchronized(properties){
			try {
				loadProperties();
			}
			catch(Exception e){
				loadDefaults();
			}
		}
		
		Runtime.getRuntime().addShutdownHook(new Thread(){
			public void run(){
				try {
					storeProperties();
				}
				catch(IOException e){}
			}
		});		
	}
	
	private static void loadDefaults(){
		properties.setProperty(DB_PATH, "");
		properties.setProperty(DB_MANAGER, "de.ba_leipzig.discoinv.db.SQLiteDBManager");
	}
	
	private static void loadProperties() throws IOException {
		synchronized(file){
			// Konfiguration einlesen
			BufferedReader reader = new BufferedReader(new FileReader(file));
			properties.load(reader);
			reader.close();
			
			// Eingelesene Konfiguration berprfen
			if(!configOK()){
				throw new IOException();
			}
		}
	}
	
	private static boolean configOK(){
		String dbPath = properties.getProperty(DB_PATH);
		String dbManager = properties.getProperty(DB_MANAGER);
		return dbPath != null && dbManager != null;
	}
	
	private static void storeProperties() throws IOException {
		synchronized(file){
			BufferedWriter writer = new BufferedWriter(new FileWriter(file));
			properties.store(writer, null);
			writer.close();
		}
	}
	
	public String getDBManager(){
		synchronized(properties){
			return properties.getProperty(DB_MANAGER);
		}
	}
	
	public String getDBPath(){
		synchronized(properties){
			return properties.getProperty(DB_PATH);
		}
	}
	
	public void setDBPath(String dbPath) throws IOException{
		synchronized(properties){
			if(dbPath != getDBPath()){
				properties.setProperty(DB_PATH, dbPath);
				storeProperties();
			}
		}
	}
}