package de.ba_leipzig.discoinv;

import java.awt.*;
import java.awt.event.*;
import java.awt.print.PrinterException;
import java.io.*;
import java.sql.SQLException;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.JTable.PrintMode;
import javax.swing.filechooser.FileFilter;

import de.ba_leipzig.discoinv.db.*;
import de.ba_leipzig.discoinv.dialogs.*;
import de.ba_leipzig.discoinv.tables.*;
import de.ba_leipzig.discoinv.tree.*;
import de.ba_leipzig.discoinv.tree.TreeCellEditor;
import de.ba_leipzig.discoinv.tree.TreeModel;
import de.ba_leipzig.discoinv.tree.TreeNode;
import javax.swing.JMenuItem;

public class DiscoInv extends JFrame {
	
	private class ExitHandler extends WindowAdapter implements ActionListener {
		public void windowClosing(WindowEvent e){
			exit();
		}
		
		public void actionPerformed(ActionEvent e){
			exit();
		}
		
		private void exit(){
			try {
				DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
				if(dbManager.isOpenedDB()){
					dbManager.closeDB();
				}
			}
			catch(Exception e){
				System.exit(-1);
			}
			
			System.exit(0);
		}
	}
	
	private class PrintHandler implements ActionListener {
		private JFrame parent;
		
		public PrintHandler(JFrame parent){
			if(parent == null){
				throw new IllegalArgumentException("parent must not be null!");
			}
			
			this.parent = parent;
		}
		
		public void actionPerformed(ActionEvent event){
			try {
				System.out.println("Tabelle drucken");
				jContentTable.print(PrintMode.FIT_WIDTH);
			}
			catch(PrinterException e){
				new ExceptionHandler(parent).handle(e, "Fehler beim Drucken",
						"Beim Drucken der Tabelle ist ein Fehler aufgetreten.");
			}
		}
	}
	
	private abstract class DBFileHandler implements ActionListener {
		protected JFrame parent;
		protected boolean firstOpen = true;
				
		public DBFileHandler(JFrame parent){
			this.parent = parent;
		}
		
		public void actionPerformed(ActionEvent event){
			String path = config.getDBPath();
			JFileChooser chooser = new JFileChooser(path);
			chooser.addChoosableFileFilter(new SQLiteFileFilter());
			chooser.setAcceptAllFileFilterUsed(false);
			chooser.setSelectedFile(new File(path));
			
			try {
				doActionPerformed(chooser);
			}
			catch(DBManagerLoadException e){
				new ExceptionHandler(parent).handle(e);
			}
		}
		
		protected void handleNoFileChosen(boolean isNewCalling){
			String[] optionsCaption = new String[3];
			
			if(isNewCalling){
				optionsCaption[0] =
					"Mchten Sie eine bestehende Datenbank ffnen, eine" +
					" neue anlegen oder das Programm beenden?";
				
				optionsCaption[1] =
					"Klicken Sie auf \"Ja\" um eine Datenbank zu ffnen.";
				
				optionsCaption[2] =
					"Klicken Sie auf \"Nein\" um eine neue Datenbank anzulegen.";
			}
			else {
				optionsCaption[0] =
					"Mchten Sie eine neue Datenbank anlegen, eine bestehende" +
					" ffnen oder das Programm beenden?";
				
				optionsCaption[1] =
					"Klicken Sie auf \"Ja\" um eine neue Datenbank anzulegen.";
				
				optionsCaption[2] =
					"Klicken Sie auf \"Nein\" um eine Datenbank zu ffnen.";					
			}
			
			int option = JOptionPane.showConfirmDialog(parent, new String[]{
					"Das Programm kann nicht arbeiten, da keine Datenbank ausgewhlt wurde.",
					optionsCaption[0],
					" ",
					optionsCaption[1],
					optionsCaption[2],
					"Klicken Sie auf \"Abbrechen\" um das Programm zu beenden"
					},
					"DiscoInv",
					JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
			
			if(option == JOptionPane.YES_OPTION){
				if(isNewCalling){
					openDBHandler.actionPerformed(null);
				}
				else {
					newDBHandler.actionPerformed(null);					
				}
			}
			else if(option == JOptionPane.NO_OPTION){
				if(isNewCalling){
					newDBHandler.actionPerformed(null);
				}
				else {
					openDBHandler.actionPerformed(null);
				}
			}
			else {
				System.exit(0);
			}
		}
		
		protected abstract void doActionPerformed(JFileChooser chooser)
		throws DBManagerLoadException;
	}
	
	private class NewDBHandler extends DBFileHandler {
		public NewDBHandler(JFrame parent) {
			super(parent);
		}
		
		protected void doActionPerformed(JFileChooser chooser)
		throws DBManagerLoadException {
			chooser.setSelectedFile(new File(""));
			chooser.setDialogTitle("Neu");
			chooser.setApproveButtonText("Neu");
			chooser.setApproveButtonToolTipText("Angegebene Datei neu anlegen");
			chooser.setApproveButtonMnemonic(KeyEvent.VK_N);
			
			if(chooser.showOpenDialog(parent) == JFileChooser.APPROVE_OPTION){
				Cursor cursor = parent.getCursor();
				parent.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				
				try {
					// Get chosen file
					File file = chooser.getSelectedFile();
					
					// Add file extension if necessary
					String fileName = file.getAbsolutePath();
					String fileExtension =
						new SQLiteFileFilter().getFileExtension();
					String regex = "^.*?" + fileExtension + "$";
					
					if(!fileName.matches(regex)){
						file = new File(fileName + fileExtension);
					}
					
					if(!file.exists()){
						// Init Managers
						DatabaseManager dbManager =
							DBManagerFactory.getDatabaseManager();
						dbManager.newDB(file);
						tableManager.setDatabaseManager(dbManager);
						
						if(treeManager.isRootSelected()){
							// Reload root table
							tableManager.showTableForPath(
									treeManager.getRootPath());
						}
						else {
							// Set initial focus to tree root
							// Root table will be reloaded automatically
							treeManager.selectRoot();
						}
						
						// Save file path to configuration
						config.setDBPath(file.getAbsolutePath());
						
						// Set application title
						parent.setTitle(APP_TITLE + " - " + file.getName());						
					}
					else {
						JOptionPane.showMessageDialog(parent, 
								"Die angegebene Datei existiert bereits.\n" +
								"Bitte whlen Sie eine andere aus!",
								APP_TITLE + " - Neue Datei",
								JOptionPane.WARNING_MESSAGE);
						actionPerformed(null);
					}
				}
				catch(IOException e){
					new ExceptionHandler(parent).handle(e,
							"Konfigurationsfehler",
							"Die gewhlte Datei konnte nicht in der " +
							"Konfiguration gespeichert werden.",
							"Bitte berprfen Sie die Konfigurationsdatei.");
				}
				catch(SQLException e){
					new ExceptionHandler(parent).handle(e, "Beim Erstellen " +
							"der Datenbank ist ein Problem aufgetreten:");
				}
				finally {
					parent.setCursor(cursor);
				}
			}
			else if(firstOpen){
				handleNoFileChosen(true);
			}
			
			firstOpen = false;
		}
	}
	
	private class OpenDBHandler extends DBFileHandler {
		public OpenDBHandler(JFrame parent){
			super(parent);
		}
		
		protected void doActionPerformed(JFileChooser chooser)
		throws DBManagerLoadException {
			chooser.setApproveButtonMnemonic('');
			if(chooser.showOpenDialog(parent) == JFileChooser.APPROVE_OPTION){
				Cursor cursor = parent.getCursor();
				parent.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				
				try {
					// Get chosen file
					File file = chooser.getSelectedFile();
					
					if(file.exists()){
						// Init Managers
						DatabaseManager dbManager =
							DBManagerFactory.getDatabaseManager();
						dbManager.openDB(file);
						tableManager.setDatabaseManager(dbManager);
						
						if(treeManager.isRootSelected()){
							// Reload root table
							tableManager.showTableForPath(
									treeManager.getRootPath());
						}
						else {
							// Set initial focus to tree root
							// Root table will be reloaded automatically
							treeManager.selectRoot();
						}
						
						// Save file path to configuration
						config.setDBPath(file.getAbsolutePath());
						
						// Set application title
						parent.setTitle(APP_TITLE + " - " + file.getName());						
					}
					else {
						JOptionPane.showMessageDialog(parent, 
								"Die angegebene Datei existiert nicht.\n" +
								"Bitte whlen Sie eine andere aus!", 
								APP_TITLE + " - Datei ffnen",
								JOptionPane.WARNING_MESSAGE);
						actionPerformed(null);
					}
				}
				catch(IOException e){
					new ExceptionHandler(parent).handle(e,
							"Konfigurationsfehler",
							"Die gewhlte Datei konnte nicht in der " +
							"Konfiguration gespeichert werden.",
							"Bitte berprfen Sie die Konfigurationsdatei.");
				}
				catch(SQLException e){
					new ExceptionHandler(parent).handle(e, "Beim ffnen " +
							"der Datenbank ist ein Problem aufgetreten:");
				}
				catch(DBIntegrityException e){
					new ExceptionHandler(parent).handle(e, "Datenbankfehler",
							e.getMessage(), "Bitte ffnen Sie eine korrekte Datei!");
					actionPerformed(null);
				}
				finally {
					parent.setCursor(cursor);
				}
			}
			else if(firstOpen){
				handleNoFileChosen(false);
			}
			
			firstOpen = false;
		}
	}
	
	private class SaveDBAsHandler extends DBFileHandler {
		public SaveDBAsHandler(JFrame parent){
			super(parent);
		}
		
		protected void doActionPerformed(JFileChooser chooser)
		throws DBManagerLoadException {
			chooser.setDialogTitle("Speichern unter");
			chooser.setApproveButtonMnemonic(KeyEvent.VK_S);
			chooser.setApproveButtonToolTipText("Datenbank in angegebener " +
					"Datei speichern");
			
			if(chooser.showSaveDialog(parent) == JFileChooser.APPROVE_OPTION){
				Cursor cursor = parent.getCursor();
				parent.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				
				try {
					File file = chooser.getSelectedFile();
					
					// Add file extension if necessary
					String fileName = file.getAbsolutePath();
					String fileExtension =
						new SQLiteFileFilter().getFileExtension();
					String regex = "^.*?" + fileExtension + "$";
					
					if(!fileName.matches(regex)){
						file = new File(fileName + fileExtension);
					}
					
					if(!file.exists()){
						// Save database in file
						saveDBAs(file);
					}
					else {
						int choice = JOptionPane.showConfirmDialog(parent, 
								"Die angegebene Datei existiert bereits.\n" +
								"Soll sie berschrieben werden?", 
								" - Datei speichern unter",
								JOptionPane.YES_NO_CANCEL_OPTION,
								JOptionPane.QUESTION_MESSAGE);
						
						if(choice == JOptionPane.YES_OPTION){
							saveDBAs(file);
						}
						else if(choice == JOptionPane.NO_OPTION){
							actionPerformed(null);
						}
					}
				}
				finally {
					parent.setCursor(cursor);
				}
			}
		}
		
		private void saveDBAs(File file) throws DBManagerLoadException {
			try {
				DatabaseManager dbManager =
					DBManagerFactory.getDatabaseManager();
				dbManager.saveDBAs(file);
				
				// Save file path to configuration
				config.setDBPath(file.getAbsolutePath());
				
				// Set application title
				parent.setTitle(APP_TITLE + " - " + file.getName());
			}
			catch(IOException e){
				new ExceptionHandler(parent).handle(e,
						"Konfigurationsfehler",
						"Die gewhlte Datei konnte nicht in der " +
						"Konfiguration gespeichert werden.",
						"Bitte berprfen Sie die Konfigurationsdatei.");
			}
		}
	}
	
	private class SQLiteFileFilter extends FileFilter {
		public boolean accept(File file) {
			return file.getAbsolutePath().endsWith(".s3db") ||
					file.isDirectory();
		}
		
		public String getDescription() {
			return "SQLite Dateien (*" + getFileExtension() + ")";
		}
		
		public String getFileExtension(){
			return ".s3db";
		}
	}
	
	public static abstract class AbstractEditHandler implements ActionListener {
		private static final String msg =
			"Beim Neuaufbau des Objektbaums ist ein Datenbankfehler aufgetreten:";
		
		protected JFrame parent;
		
		public AbstractEditHandler(JFrame parent){
			this.parent = parent;
		}
		
		public void actionPerformed(ActionEvent event){
			Cursor cursor = parent.getCursor();
			parent.setCursor(new Cursor(Cursor.WAIT_CURSOR));
			AbstractDialog dialog = getDialog();
			parent.setCursor(cursor);
			
			dialog.setModalityType(Dialog.ModalityType.DOCUMENT_MODAL);
			dialog.setVisible(true);
			
			if(dialog.getExitType() == AbstractDialog.OK){
				cursor = parent.getCursor();
				parent.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				
				try { 
					doActionPerformed();
				}			
				catch(DBManagerLoadException e){
					new ExceptionHandler(parent).handle(e);
				}
				catch(SQLException e){
					new ExceptionHandler(parent).handle(e, msg);
				}
				finally {
					parent.setCursor(cursor);
				}
			}
		}
		
		protected abstract void doActionPerformed()
			throws DBManagerLoadException, SQLException;
		
		protected abstract AbstractDialog getDialog();
	}
	
	private class AddPlaceHandler extends AbstractEditHandler {
		public AddPlaceHandler(JFrame parent){
			super(parent);
		}
		
		protected void doActionPerformed()
		throws DBManagerLoadException, SQLException {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			treeManager.loadPlaces(dbManager);
			treeManager.selectPlaces(true);
			tableManager.showTableForPath(treeManager.getPlacesPath());
		}

		protected AbstractDialog getDialog() {
			return new AddPlaceDialog(parent);
		}
	}
	
	private class AddMiscHandler extends AbstractEditHandler {
		public AddMiscHandler(JFrame parent){
			super(parent);
		}
		
		protected void doActionPerformed()
		throws DBManagerLoadException, SQLException {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			treeManager.loadMisc(dbManager);
			treeManager.selectMisc(true);
			tableManager.showTableForPath(treeManager.getMiscPath());
		}

		protected AbstractDialog getDialog() {
			return new AddMiscDialog(parent);
		}
	}
	
	private class AddBeverageHandler extends AbstractEditHandler {
		public AddBeverageHandler(JFrame parent){
			super(parent);
		}
		
		protected void doActionPerformed()
		throws DBManagerLoadException, SQLException {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			treeManager.loadBeverages(dbManager);
			treeManager.selectBeverages(true);
			tableManager.showTableForPath(treeManager.getBeveragesPath());
		}

		protected AbstractDialog getDialog() {
			return new AddBeverageDialog(parent);
		}
	}	
	
	private class InfoHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JOptionPane.showMessageDialog(null, new String[]{
					"DiscoInv: Inventarverwaltung fr Diskotheken",
					"(C) 2007, Christian Grfe & Matthias Rothe"},
					"DiscoInv - Info", JOptionPane.INFORMATION_MESSAGE);
		}
	}
	
	private class EditMenu extends JMenu {
		private static final long serialVersionUID = 5206524136483818917L;
		private JFrame parent;
		
		public EditMenu(JFrame parent, String title){
			super(title);
			
			if(parent == null){
				throw new IllegalArgumentException("parent must not be null!");
			}
			
			this.parent = parent;
		}
		
		private JLabel getNoOpMenuItem(){
			JLabel noOperationMenuItem =
				new JLabel("      keine Operation verfgbar      ");
			noOperationMenuItem.setEnabled(false);
			
			return noOperationMenuItem;
		}
		
		private void createMenuHeader(TreeElement element, JPopupMenu popupMenu){
			JLabel header = new JLabel("   " + element.toString() + ":");
			popupMenu.add(header);
			popupMenu.addSeparator();
		}
		
		private JPopupMenu createPopupMenu(JPopupMenu popupMenu){
			popupMenu.removeAll();
			
			TreePath path = jObjectTree.getSelectionPath();
			TreeNode node =	(TreeNode)path.getLastPathComponent();
			
			createMenuHeader(node.getUserObject(), popupMenu);
			
			if(path.getPathCount() == 3){
				EditPopupMenu epm =
					new EditPopupMenu(parent, node, treeManager, tableManager,
										false);
				epm.setupPopupMenu(popupMenu);
			}
			else {
				popupMenu.add(getNoOpMenuItem());
			}
			
			return popupMenu;
		}
		
		public JPopupMenu getPopupMenu(){
			return createPopupMenu(super.getPopupMenu());
	    }
	}
	
	private class PopupHandler extends MouseAdapter {
		private JFrame parent;
		private JMenuItem addPlaceMenuItem;
		private JMenuItem addBeverageMenuItem;
		private JMenuItem addMiscMenuItem;
		
		public PopupHandler(JFrame parent){
			if(parent == null){
				throw new IllegalArgumentException("parent must not be null!");
			}
			
			this.parent = parent;
		}
		
		public void mousePressed(MouseEvent e){
			if(e.isPopupTrigger()){
				showPopup(e);
			}
		}
		
		public void mouseReleased(MouseEvent e){
			if(e.isPopupTrigger()){
				showPopup(e);
			}					
		}
		
		private void showPopup(MouseEvent e){
			final int x = e.getX();
			final int y = e.getY();
			
			final TreePath path = jObjectTree.getPathForLocation(x, y);
			jObjectTree.setSelectionPath(path);
			
			Thread t = new Thread(new Runnable(){
				public void run() {
					TreeNode node =	(TreeNode)path.getLastPathComponent();
					JPopupMenu menu = null;
					
					menu = (path.getPathCount() < 3) ? 
							getPopupMenu(node, x, y) : 
								new EditPopupMenu(parent, node, treeManager,
													tableManager);
					
					jObjectTree.add(menu);
					menu.show(jObjectTree, x, y);
				}
			}, "Popup");
			t.start();
		}
		
		private JPopupMenu getPopupMenu(TreeNode node, int x, int y){
			JPopupMenu menu = new JPopupMenu();
			
			if(node == treeManager.getRootNode()){
				menu.add(getAddPlaceMenuItem());
				menu.add(getAddBeverageMenuItem());
				menu.add(getAddMiscMenuItem());
			}
			else if(node == treeManager.getPlacesNode()){
				menu.add(getAddPlaceMenuItem());
			}
			else if(node == treeManager.getBeveragesNode()){
				menu.add(getAddBeverageMenuItem());
			}
			else if(node == treeManager.getMiscNode()){
				menu.add(getAddMiscMenuItem());
			}
			
			return menu;
		}
		
		private JMenuItem getAddPlaceMenuItem(){
			if(addPlaceMenuItem == null){
				addPlaceMenuItem =
					new JMenuItem("Standort hinzufgen", KeyEvent.VK_O);
				addPlaceMenuItem.addActionListener(
						new AddPlaceHandler(parent));
			}
			return addPlaceMenuItem;
		}
		
		private JMenuItem getAddBeverageMenuItem(){
			if(addBeverageMenuItem == null){
				addBeverageMenuItem =
					new JMenuItem("Getrnk hinzufgen", KeyEvent.VK_G);
				addBeverageMenuItem.addActionListener(
						new AddBeverageHandler(parent));
			}
			return addBeverageMenuItem;
		}
		
		private JMenuItem getAddMiscMenuItem(){
			if(addMiscMenuItem == null){
				addMiscMenuItem =
					new JMenuItem("Sonstiges hinzufgen", KeyEvent.VK_S);
				addMiscMenuItem.addActionListener(new AddMiscHandler(parent));
			}
			return addMiscMenuItem;
		}
	}
	
	private class TreeDeleteKeyHandler extends KeyAdapter {
		private JFrame parent;
		
		public TreeDeleteKeyHandler(JFrame parent){
			if(parent == null){
				throw new IllegalArgumentException("parent must not be null!");
			}
			
			this.parent = parent;
		}
		
		public void keyReleased(KeyEvent event){
			if(event.getKeyCode() == KeyEvent.VK_DELETE){
				TreePath path = treeManager.getSelectedPath();
				if(treeManager.isDeleteableNodePath(path)){
					TreeNode node = (TreeNode)path.getLastPathComponent();
					new EditPopupMenu.DeleteHandler(parent, treeManager, node)
					.actionPerformed(null);					
				}
			}
		}
	}
	
	private class AcceleratorHandler extends KeyAdapter {
		private JFrame parent;
		
		public AcceleratorHandler(JFrame parent){
			if(parent == null){
				throw new IllegalArgumentException("parent must not be null!");
			}
			
			this.parent = parent;
		}
		
		public void keyReleased(KeyEvent event){
			if(event.isControlDown()){
				handleCtrlAccelerators(event);
			}
			else if(event.getKeyCode() == KeyEvent.VK_F1){
				new InfoHandler().actionPerformed(null);
			}
		}
		
		private void handleCtrlAccelerators(KeyEvent event){
			if(event.isShiftDown()){
				handleCtrlShiftAccelerators(event);
			}
			else {
				if(event.getKeyCode() == KeyEvent.VK_N){
					new NewDBHandler(parent).actionPerformed(null);
				}
				else if(event.getKeyCode() == KeyEvent.VK_O){
					new OpenDBHandler(parent).actionPerformed(null);
				}
				else if(event.getKeyCode() == KeyEvent.VK_S){
					new SaveDBAsHandler(parent).actionPerformed(null);
				}
				else if(event.getKeyCode() == KeyEvent.VK_P){
					new PrintHandler(parent).actionPerformed(null);
				}
			}
		}
		
		private void handleCtrlShiftAccelerators(KeyEvent event){
			if(event.getKeyCode() == KeyEvent.VK_O){
				new AddPlaceHandler(parent).actionPerformed(null);
			}
			else if(event.getKeyCode() == KeyEvent.VK_G){
				new AddBeverageHandler(parent).actionPerformed(null);
			}
			else if(event.getKeyCode() == KeyEvent.VK_S){
				new AddMiscHandler(parent).actionPerformed(null);
			}
		}
	}
	
	private static final long serialVersionUID = 1L;
	private static final String APP_TITLE = "DiscoInv";
	private final int APP_HEIGHT;
	private final int APP_WIDTH;
	private JPanel jContentPane = null;
	private JMenuBar jJMenuBar = null;
	private JMenu jDBMenu = null;
	private JSplitPane jMainPane = null;
	private JScrollPane jTreeScrollPane = null;
	private JScrollPane jTableScrollPane = null;
	private CustomJTable jContentTable = null;
	private JTree jObjectTree;
	private JMenu jEditMenu = null;
	private JMenu jAddMenu = null;
	private JMenu jHelpMenu = null;
	private JMenuItem jExitMenuItem = null;
	private JMenuItem jSaveMenuItem = null;
	private JMenuItem jOpenMenuItem = null;
	private TableManager tableManager = new TableManager();  //  @jve:decl-index=0:
	private TreeManager treeManager = new TreeManager();  //  @jve:decl-index=0:
	private ExitHandler exitHandler = new ExitHandler();  //  @jve:decl-index=0:
	private OpenDBHandler openDBHandler = new OpenDBHandler(this);  //  @jve:decl-index=0:
	private NewDBHandler newDBHandler = new NewDBHandler(this);
	private Config config = new Config();  //  @jve:decl-index=0:
	private JMenuItem jInfoMenuItem = null;
	private JMenuItem jAddPlaceMenuItem = null;
	private JMenuItem jAddBeverageMenuItem = null;
	private JMenuItem jAddMiscMenuItem = null;
	private JMenuItem jNewMenuItem = null;
	private JMenuItem jPrintMenuItem = null;
	
	/**
	 * This method initializes jJMenuBar	
	 * 	
	 * @return javax.swing.JMenuBar	
	 */
	private JMenuBar getJJMenuBar() {
		if (jJMenuBar == null) {
			jJMenuBar = new JMenuBar();
			jJMenuBar.add(getJDBMenu());
			jJMenuBar.add(getJEditMenu());
			jJMenuBar.add(getJAddMenu());
			jJMenuBar.add(getJHelpMenu());
		}
		return jJMenuBar;
	}

	/**
	 * This method initializes jDBMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getJDBMenu() {
		if (jDBMenu == null) {
			jDBMenu = new JMenu("Datenbank");
			jDBMenu.setMnemonic('d');
			jDBMenu.add(getJNewMenuItem());
			jDBMenu.add(getJOpenMenuItem());
			jDBMenu.add(getJSaveMenuItem());
			jDBMenu.addSeparator();
			jDBMenu.add(getJPrintMenuItem());
			jDBMenu.addSeparator();
			jDBMenu.add(getJExitMenuItem());
		}
		return jDBMenu;
	}

	/**
	 * This method initializes jMainPane	
	 * 	
	 * @return javax.swing.JSplitPane	
	 */
	private JSplitPane getJMainPane() {
		if (jMainPane == null) {
			jMainPane = new JSplitPane();
			jMainPane.setDividerLocation(200);
			jMainPane.setLeftComponent(getJTreeScrollPane());
			jMainPane.setRightComponent(getJTableScrollPane());
		}
		return jMainPane;
	}

	/**
	 * This method initializes jTreeScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJTreeScrollPane() {
		if (jTreeScrollPane == null) {
			jTreeScrollPane = new JScrollPane();
			jTreeScrollPane.setViewportView(getJObjectTree());
		}
		return jTreeScrollPane;
	}

	/**
	 * This method initializes jTableScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJTableScrollPane() {
		if (jTableScrollPane == null) {
			jTableScrollPane = new JScrollPane();
			jTableScrollPane.setViewportView(getJContentTable());
		}
		return jTableScrollPane;
	}

	/**
	 * This method initializes jContentTable	
	 * 	
	 * @return javax.swing.JTable	
	 */
	private JTable getJContentTable() {
		if (jContentTable == null){
			jContentTable = new CustomJTable(new GenericTableModel());
			jContentTable.setFillsViewportHeight(true);
			jContentTable.setAutoCreateRowSorter(true);
			tableManager.setContentTable(jContentTable);
		}
		return jContentTable;
	}
	
	/**
	 * This method initializes jObjectTree	
	 * 	
	 * @return javax.swing.JTree	
	 */
	private JTree getJObjectTree() {
		if (jObjectTree == null) {
			TreeModel model = new TreeModel();
			model.addTreePathValueChangeListener(new RenameNodeHandler(this));
			
			jObjectTree = new JTree(model);
			jObjectTree.setEditable(true);
			jObjectTree.setCellEditor(new TreeCellEditor(jObjectTree,
					(DefaultTreeCellRenderer)jObjectTree.getCellRenderer()));
			jObjectTree.getSelectionModel().
					setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
			jObjectTree.addTreeSelectionListener(
					new TreeSelectionHandler(this, tableManager));
			jObjectTree.addMouseListener(new PopupHandler(this));
			jObjectTree.addKeyListener(new TreeDeleteKeyHandler(this));
			jObjectTree.addTreeWillExpandListener(new TreeLoadHandler(this, treeManager));
			
			treeManager.setObjectTree(jObjectTree);
		}
		return jObjectTree;
	}

	/**
	 * This method initializes jEditMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getJEditMenu() {
		if (jEditMenu == null) {
			jEditMenu = new EditMenu(this, "Bearbeiten");
			jEditMenu.setMnemonic('b');
		}
		return jEditMenu;
	}

	/**
	 * This method initializes jAddMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getJAddMenu() {
		if (jAddMenu == null) {
			jAddMenu = new JMenu("Hinzufgen");
			jAddMenu.setMnemonic('f');
			jAddMenu.add(getJAddPlaceMenuItem());
			jAddMenu.add(getJAddBeverageMenuItem());
			jAddMenu.add(getJAddMiscMenuItem());
		}
		return jAddMenu;
	}

	/**
	 * This method initializes jHelpMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getJHelpMenu() {
		if (jHelpMenu == null) {
			jHelpMenu = new JMenu("Hilfe");
			jHelpMenu.setMnemonic('h');
			jHelpMenu.add(getJInfoMenuItem());
		}
		return jHelpMenu;
	}

	/**
	 * This method initializes jExitMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJExitMenuItem() {
		if (jExitMenuItem == null) {
			jExitMenuItem = new JMenuItem("Beenden");
			jExitMenuItem.setMnemonic('b');
			KeyStroke keyStroke =
				KeyStroke.getKeyStroke(KeyEvent.VK_F4, KeyEvent.ALT_DOWN_MASK);
			jExitMenuItem.setAccelerator(keyStroke);
			jExitMenuItem.addActionListener(exitHandler);
		}
		return jExitMenuItem;
	}

	/**
	 * This method initializes jSaveMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJSaveMenuItem() {
		if (jSaveMenuItem == null) {
			jSaveMenuItem = new JMenuItem("Speichern unter");
			jSaveMenuItem.setMnemonic('s');
			KeyStroke keyStroke =
				KeyStroke.getKeyStroke(KeyEvent.VK_S, KeyEvent.CTRL_DOWN_MASK);
			jSaveMenuItem.setAccelerator(keyStroke);
			jSaveMenuItem.addActionListener(new SaveDBAsHandler(this));
		}
		return jSaveMenuItem;
	}

	/**
	 * This method initializes jOpenMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJOpenMenuItem() {
		if (jOpenMenuItem == null) {
			jOpenMenuItem = new JMenuItem("ffnen");
			jOpenMenuItem.setMnemonic('');
			KeyStroke keyStroke =
				KeyStroke.getKeyStroke(KeyEvent.VK_O, KeyEvent.CTRL_DOWN_MASK);
			jOpenMenuItem.setAccelerator(keyStroke);
			jOpenMenuItem.addActionListener(openDBHandler);
		}
		return jOpenMenuItem;
	}

	/**
	 * This method initializes jNewMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJNewMenuItem() {
		if (jNewMenuItem == null) {
			jNewMenuItem = new JMenuItem("Neu");
			jNewMenuItem.setMnemonic('n');
			KeyStroke keyStroke =
				KeyStroke.getKeyStroke(KeyEvent.VK_N, KeyEvent.CTRL_DOWN_MASK);
			jNewMenuItem.setAccelerator(keyStroke);
			jNewMenuItem.addActionListener(newDBHandler);
		}
		return jNewMenuItem;
	}

	/**
	 * This method initializes jPrintMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJPrintMenuItem() {
		if (jPrintMenuItem == null) {
			jPrintMenuItem = new JMenuItem("Drucken");
			jPrintMenuItem.setMnemonic('d');
			KeyStroke keyStroke =
				KeyStroke.getKeyStroke(KeyEvent.VK_P, KeyEvent.CTRL_DOWN_MASK);
			jPrintMenuItem.setAccelerator(keyStroke);
			jPrintMenuItem.setToolTipText("Aktuelle Tabelle drucken");
			jPrintMenuItem.addActionListener(new PrintHandler(this));
		}
		return jPrintMenuItem;
	}

	/**
	 * This method initializes jInfoMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJInfoMenuItem() {
		if (jInfoMenuItem == null) {
			jInfoMenuItem = new JMenuItem("Info");
			jInfoMenuItem.setMnemonic('i');
			KeyStroke keyStroke =
				KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0);
			jInfoMenuItem.setAccelerator(keyStroke);
			jInfoMenuItem.addActionListener(new InfoHandler());
		}
		return jInfoMenuItem;
	}

	/**
	 * This method initializes jAddPlaceMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJAddPlaceMenuItem() {
		if (jAddPlaceMenuItem == null) {
			jAddPlaceMenuItem = new JMenuItem("Standort");
			jAddPlaceMenuItem.setMnemonic('o');
			KeyStroke keyStroke =
				KeyStroke.getKeyStroke(KeyEvent.VK_O,
						KeyEvent.CTRL_DOWN_MASK | KeyEvent.SHIFT_DOWN_MASK);
			jAddPlaceMenuItem.setAccelerator(keyStroke);
			jAddPlaceMenuItem.addActionListener(new AddPlaceHandler(this));
		}
		return jAddPlaceMenuItem;
	}

	/**
	 * This method initializes jAddBeverageMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJAddBeverageMenuItem() {
		if (jAddBeverageMenuItem == null) {
			jAddBeverageMenuItem = new JMenuItem("Getrnk");
			jAddBeverageMenuItem.setMnemonic('g');
			KeyStroke keyStroke =
				KeyStroke.getKeyStroke(KeyEvent.VK_G,
						KeyEvent.CTRL_DOWN_MASK | KeyEvent.SHIFT_DOWN_MASK);
			jAddBeverageMenuItem.setAccelerator(keyStroke);
			jAddBeverageMenuItem.addActionListener(
					new AddBeverageHandler(this));
		}
		return jAddBeverageMenuItem;
	}

	/**
	 * This method initializes jAddMiscMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJAddMiscMenuItem() {
		if (jAddMiscMenuItem == null) {
			jAddMiscMenuItem = new JMenuItem("Sonstiges");
			jAddMiscMenuItem.setMnemonic('s');
			KeyStroke keyStroke =
				KeyStroke.getKeyStroke(KeyEvent.VK_S,
						KeyEvent.CTRL_DOWN_MASK | KeyEvent.SHIFT_DOWN_MASK);
			jAddMiscMenuItem.setAccelerator(keyStroke);
			jAddMiscMenuItem.addActionListener(new AddMiscHandler(this));
		}
		return jAddMiscMenuItem;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new DiscoInv();
			}
		});
	}

	/**
	 * This is the default constructor
	 */
	public DiscoInv() {
		super();
		
		Dimension screenSize = this.getToolkit().getScreenSize();
		APP_HEIGHT = screenSize.height - 50;
		APP_WIDTH = screenSize.width - 50;
		
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		// Set Look & Feel
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch(Exception e){}
		
		// Set properties
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(exitHandler);
		this.setSize(APP_WIDTH, APP_HEIGHT);
		this.setTitle(APP_TITLE);
		
		// Add accelerator handler
		this.addKeyListener(new AcceleratorHandler(this));
		
		// Position application window in the middle of the screen
		Dimension screenSize = this.getToolkit().getScreenSize();
		Point location = new Point();
		location.x = (int)((screenSize.getWidth() / 2) - (APP_WIDTH / 2));
		location.y = (int)((screenSize.getHeight() / 2) - (APP_HEIGHT / 2));
		this.setLocation(location);
		
		// Add content
		this.setJMenuBar(getJJMenuBar());
		this.setContentPane(getJContentPane());
		
		// Set main window visible
		this.setVisible(true);
		
		// Try to open database
		if(new Config().getDBPath().equals("")){
			handleEmptyDBPath();
		}
		else {
			openDBHandler.actionPerformed(null);
		}
	}
	
	private void handleEmptyDBPath(){
		int option = JOptionPane.showConfirmDialog(this, new String[]{
				"Es konnte keine Standarddatenbank gefunden werden.",
				"Mchten Sie eine neue Datenbank anlegen oder eine bestehende ffnen?",
				" ",
				"Klicken Sie auf \"Ja\" um eine neue Datenbank anzulegen.",
				"Klicken Sie auf \"Nein\" um eine Datenbank zu ffnen."
				},
				"DiscoInv",
				JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
		
		if(option == JOptionPane.YES_OPTION){
			newDBHandler.actionPerformed(null);
		}
		else {
			openDBHandler.actionPerformed(null);
		}
	}
	
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJMainPane(), BorderLayout.CENTER);
		}
		return jContentPane;
	}
}