package de.ba_leipzig.discoinv;

import java.awt.*;
import java.awt.event.*;
import java.sql.SQLException;

import javax.swing.*;

import de.ba_leipzig.discoinv.db.*;
import de.ba_leipzig.discoinv.dialogs.*;
import de.ba_leipzig.discoinv.tables.*;
import de.ba_leipzig.discoinv.tree.*;

public class EditPopupMenu extends JPopupMenu {
	
	static class DeleteHandler implements ActionListener {
		private TreeNode node;
		private TreeManager treeManager;
		private Component parent;
		
		DeleteHandler(Component parent, TreeManager treeManager, TreeNode node){
			if(parent == null || treeManager == null || node == null){
				throw new IllegalArgumentException("parameters must not be null!");
			}
			
			this.parent = parent;
			this.treeManager = treeManager;
			this.node = node;
		}
		
		public void actionPerformed(ActionEvent event){
			TreeElement element = node.getUserObject();
			String message = "Soll ";
			String title = "";
			
			switch(element.getType()){
				case place:
					message += "der Standort ";
					title += "Standort";
					break;
				case beverage:
					message += "das Getrnk ";
					title += "Getrnk";
					break;
				case misc:
					message += "das Sonstige ";
					title += "Sonstiges";
					break;
			}
			
			String name = ((Nameable)element.getObject()).getName();
			message += "\"" + name + "\" wirklich gelscht werden?";
			title += " Lschen besttigen";
			
			int option = JOptionPane.showConfirmDialog(parent, message, title, 
					JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
			
			if(option == JOptionPane.YES_OPTION){
				try {
					int id = ((Identifiable)element.getObject()).getID();
					DatabaseManager dbManager =
						DBManagerFactory.getDatabaseManager();
					
					switch(element.getType()){
						case place:
							dbManager.deletePlace(id);
							treeManager.loadPlaces(dbManager);
							treeManager.selectPlaces(true);
							break;
						case beverage:
							dbManager.deleteInvObject(id);
							treeManager.loadBeverages(dbManager);
							treeManager.selectBeverages(true);
							break;
						case misc:
							dbManager.deleteInvObject(id);
							treeManager.loadMisc(dbManager);
							treeManager.selectMisc(true);
							break;
					}					
				}
				catch(DBIntegrityException e){
					String errorTitle = "Integrittsfehler";
					String[] msg = new String[]{
							name + " kann nicht gelscht werden, da noch" +
							" andere Objekte davon abhngen!",
							"Bitte lsen Sie die Abhngigkeiten auf und" +
							" versuchen Sie die Lschung danach erneut."};
					new ExceptionHandler(parent).handle(e, errorTitle, msg);
				}
				catch(DBManagerLoadException e){
					new ExceptionHandler(parent).handle(e);
				}
				catch(SQLException e){
					String msg = "Beim Lschen des Objekts ist ein" +
					" Datenbankfehler aufgetreten:";
					new ExceptionHandler(parent).handle(e, msg);
				}
			}
		}
	}
	
	private class EditPlaceHandler extends DiscoInv.AbstractEditHandler {
		private EditPlaceDialog dialog;
		private Place place;
		
		public EditPlaceHandler(JFrame parent, Place place){
			super(parent);
			
			if(place == null){
				throw new IllegalArgumentException("place must not be null!");
			}
			
			this.place = place;
		}

		protected void doActionPerformed()
		throws DBManagerLoadException, SQLException {
			JOptionPane.showMessageDialog(parent,
					"Die Daten des Standorts " + /*place.getName() +*/
					"wurden erfolgreich gendert.",
					"Datennderung: Standort",
					JOptionPane.INFORMATION_MESSAGE);
			
			TreeElement<Identifiable> element =
				new TreeElement<Identifiable>(dialog.getObject(),
						TreeObjectType.place);
			node.setUserObject(element);
			treeManager.selectNode(node);
			treeManager.nodeChanged(node);
		}

		protected AbstractDialog getDialog(){
			if(dialog == null){
				dialog = new EditPlaceDialog(parent, place);
			}
			return dialog;
		}
	}
	
	private class EditBeverageHandler extends DiscoInv.AbstractEditHandler {
		private EditBeverageDialog dialog;
		private Beverage beverage;
		
		public EditBeverageHandler(JFrame parent, Beverage beverage){
			super(parent);
			
			if(beverage == null){
				throw new IllegalArgumentException("beverage must not be null!");
			}
			
			this.beverage = beverage;
		}

		public EditBeverageHandler(JFrame parent, BeverageWithLevel beverage){
			super(parent);
			
			if(beverage == null){
				throw new IllegalArgumentException("beverage must not be null!");
			}
			
			dialog = new EditBeverageDialog(parent, beverage);
		}

		protected void doActionPerformed()
		throws DBManagerLoadException, SQLException {
			JOptionPane.showMessageDialog(parent,
					"Die Daten des Getrnks " + /*beverage.getName() +*/
					"wurden erfolgreich gendert.",
					"Datennderung: Getrnk",
					JOptionPane.INFORMATION_MESSAGE);
			
			TreeElement<Identifiable> element =
				new TreeElement<Identifiable>(dialog.getObject(),
						TreeObjectType.beverage);
			node.setUserObject(element);
			treeManager.selectNode(node);
			treeManager.nodeChanged(node);
		}

		protected AbstractDialog getDialog(){
			if(dialog == null){
				if(beverage instanceof BeverageWithLevel){
					dialog =
						new EditBeverageDialog(parent,
								(BeverageWithLevel)beverage);
				}
				else {
					dialog = new EditBeverageDialog(parent, beverage);
				}
			}
			return dialog;
		}
	}
	
	private class NewBeverageAssociationHandler
	extends DiscoInv.AbstractEditHandler {
		
		private AddBeverageAssociationDialog dialog;
		private Place place;
		
		public NewBeverageAssociationHandler(JFrame parent, Place place) {
			super(parent);
			
			if(place == null){
				throw new IllegalArgumentException("place must not be null!");
			}
			
			this.place = place;
		}

		protected void doActionPerformed()
		throws DBManagerLoadException, SQLException {
			treeManager.selectNode(node);
			tableManager.showTableForPath(treeManager.getPathForNode(node));
		}
		
		protected AbstractDialog getDialog(){
			if(dialog == null){
				dialog = new AddBeverageAssociationDialog(parent, place);
			}
			return dialog;
		}
	}
	
	private class EditBeverageAssociationHandler
	extends DiscoInv.AbstractEditHandler {
		
		private EditBeverageAssociationDialog dialog;
		private Place place;
		
		public EditBeverageAssociationHandler(JFrame parent, Place place) {
			super(parent);
			
			if(place == null){
				throw new IllegalArgumentException("place must not be null!");
			}
			
			this.place = place;
		}
		
		protected void doActionPerformed()
		throws DBManagerLoadException, SQLException {
			treeManager.selectNode(node);
			tableManager.showTableForPath(treeManager.getPathForNode(node));
		}
		
		protected AbstractDialog getDialog(){
			if(dialog == null){
				dialog = new EditBeverageAssociationDialog(parent, place);
			}
			return dialog;
		}
	}
	
	private class NewMiscAssociationHandler
	extends DiscoInv.AbstractEditHandler {
		
		private AddMiscAssociationDialog dialog;
		private Place place;
		
		public NewMiscAssociationHandler(JFrame parent, Place place) {
			super(parent);
			
			if(place == null){
				throw new IllegalArgumentException("place must not be null!");
			}
			
			this.place = place;
		}

		protected void doActionPerformed()
		throws DBManagerLoadException, SQLException {
			treeManager.selectNode(node);
			tableManager.showTableForPath(treeManager.getPathForNode(node));
		}
		
		protected AbstractDialog getDialog(){
			if(dialog == null){
				dialog = new AddMiscAssociationDialog(parent, place);
			}
			return dialog;
		}
	}
	
	private class EditMiscAssociationHandler
	extends DiscoInv.AbstractEditHandler {
		
		private EditMiscAssociationDialog dialog;
		private Place place;
		
		public EditMiscAssociationHandler(JFrame parent, Place place) {
			super(parent);
			
			if(place == null){
				throw new IllegalArgumentException("place must not be null!");
			}
			
			this.place = place;
		}
		
		protected void doActionPerformed()
		throws DBManagerLoadException, SQLException {
			treeManager.selectNode(node);
			tableManager.showTableForPath(treeManager.getPathForNode(node));
		}
		
		protected AbstractDialog getDialog(){
			if(dialog == null){
				dialog = new EditMiscAssociationDialog(parent, place);
			}
			return dialog;
		}
	}
	
	private class NewPlaceAssociationHandler
	extends DiscoInv.AbstractEditHandler {
		
		private AddPlaceAssociationDialog dialog;
		private InvObject object;
		
		public NewPlaceAssociationHandler(JFrame parent, InvObject object) {
			super(parent);
			
			if(object == null){
				throw new IllegalArgumentException("object must not be null!");
			}
			
			this.object = object;
		}

		protected void doActionPerformed()
		throws DBManagerLoadException, SQLException {
			treeManager.selectNode(node);
			tableManager.showTableForPath(treeManager.getPathForNode(node));
		}
		
		protected AbstractDialog getDialog(){
			if(dialog == null){
				dialog = new AddPlaceAssociationDialog(parent, object);
			}
			return dialog;
		}
	}
	
	private class EditPlaceAssociationHandler
	extends DiscoInv.AbstractEditHandler {
		
		private EditPlaceAssociationDialog dialog;
		private InvObject object;
		
		public EditPlaceAssociationHandler(JFrame parent, InvObject object) {
			super(parent);
			
			if(object == null){
				throw new IllegalArgumentException("object must not be null!");
			}
			
			this.object = object;
		}
		
		protected void doActionPerformed()
		throws DBManagerLoadException, SQLException {
			treeManager.selectNode(node);
			tableManager.showTableForPath(treeManager.getPathForNode(node));
		}
		
		protected AbstractDialog getDialog(){
			if(dialog == null){
				dialog = new EditPlaceAssociationDialog(parent, object);
			}
			return dialog;
		}
	}
	
	private static final long serialVersionUID = 7052982877212389360L;
	
	private static final String sqlExceptionMsg =
		"Beim Aufbau des Popupmens ist ein Datenbankfehler aufgetreten:";
	
	private JFrame parent;
	private TreeNode node;
	private TreeManager treeManager;
	private TableManager tableManager;
	private JMenuItem deleteMenuItem;
	private JMenuItem renameMenuItem;
	private JMenuItem editPlaceDataMenuItem;
	private JMenuItem editBeverageDataMenuItem;
	private JMenuItem newPlaceAssociationMenuItem;
	private JMenuItem editPlaceAssociationMenuItem;
	private JMenuItem newBeverageAssociationMenuItem;
	private JMenuItem editBeverageAssociationMenuItem;
	private JMenuItem newMiscAssociationMenuItem;
	private JMenuItem editMiscAssociationMenuItem;
	private JMenu placeAssociationMenu;
	private JMenu beverageAssociationMenu;
	private JMenu miscAssociationMenu;
	
	public EditPopupMenu(JFrame parent, TreeNode node, TreeManager
						treeManager, TableManager tableManager, boolean setup){
		if(parent == null){
			throw new IllegalArgumentException("parent must not be null!");
		}
		if(node == null){
			throw new IllegalArgumentException("node must not be null!");
		}
		if(treeManager == null){
			throw new IllegalArgumentException("treeManager must not be null!");
		}
		if(tableManager == null){
			throw new IllegalArgumentException("tableManager must not be null!");
		}
		
		this.parent = parent;
		this.node = node;
		this.treeManager = treeManager;
		this.tableManager = tableManager;
		if(setup) setupPopupMenu(this);
	}
	
	public EditPopupMenu(JFrame parent, TreeNode node, TreeManager treeManager,
						TableManager tableManager){
		this(parent, node, treeManager, tableManager, true);
	}
	
	public void setupPopupMenu(JPopupMenu popupMenu){
		commonInit(popupMenu);
		specialInit(popupMenu);
	}
	
	private void commonInit(JPopupMenu popupMenu){
		popupMenu.add(getRenameMenuItem());
		popupMenu.add(getDeleteMenuItem());
		popupMenu.addSeparator();
	}
	
	private void specialInit(JPopupMenu popupMenu){
		switch(node.getUserObject().getType()){
			case place:
				initPlacePopup(popupMenu);
				break;
			case beverage:
				initBeveragePopup(popupMenu);
				break;
			case misc:
				initMiscPopup(popupMenu);
				break;
		}
	}
	
	private void initMiscPopup(JPopupMenu popupMenu){
		popupMenu.add(getPlaceAssociationMenu());
	}
	
	private void initBeveragePopup(JPopupMenu popupMenu){
		popupMenu.add(getEditBeverageDataMenuItem());
		popupMenu.add(getPlaceAssociationMenu());
	}
	
	private void initPlacePopup(JPopupMenu popupMenu){
		popupMenu.add(getEditPlaceDataMenuItem());
		popupMenu.add(getBeverageAssociationMenu());
		popupMenu.add(getMiscAssociationMenu());
	}
	
	private JMenuItem getDeleteMenuItem(){
		if(deleteMenuItem == null){
			deleteMenuItem = new JMenuItem("Lschen", KeyEvent.VK_L);
			KeyStroke keyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0);
			deleteMenuItem.setAccelerator(keyStroke);
			deleteMenuItem.addActionListener(
					new DeleteHandler(parent, treeManager, node));
		}
		return deleteMenuItem;
	}
	
	private JMenuItem getRenameMenuItem(){
		if(renameMenuItem == null){
			renameMenuItem = new JMenuItem("Umbenennen", KeyEvent.VK_U);
			KeyStroke keyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_F2, 0);
			renameMenuItem.setAccelerator(keyStroke);
			renameMenuItem.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					treeManager.editNode(node);
				}
			});
		}
		return renameMenuItem;
	}
	
	private JMenuItem getEditBeverageDataMenuItem() {
		if(editBeverageDataMenuItem == null){
			editBeverageDataMenuItem =
				new JMenuItem("Daten bearbeiten", KeyEvent.VK_D);
			
			ActionListener beverageHandler = null;
			Object beverage = node.getUserObject().getObject();
			
			if(beverage instanceof BeverageWithLevel){
				beverageHandler = new EditBeverageHandler(parent, 
										(BeverageWithLevel)beverage);
			}
			else if(beverage instanceof Beverage){
				beverageHandler = new EditBeverageHandler(parent, 
										(Beverage)beverage);
			}
			
			editBeverageDataMenuItem.addActionListener(beverageHandler);
		}
		return editBeverageDataMenuItem;
	}

	private JMenu getPlaceAssociationMenu(){
		if(placeAssociationMenu == null){
			placeAssociationMenu = new JMenu("Zuordnung: Standorte");
			placeAssociationMenu.setMnemonic('S');
			
			if(existPlaces()){
				placeAssociationMenu.add(getNewPlaceAssociationMenuItem());
				placeAssociationMenu.add(getEditPlaceAssociationMenuItem());				
			}
			else {
				placeAssociationMenu.setEnabled(false);
			}
		}
		return placeAssociationMenu;
	}
	
	private JMenuItem getNewPlaceAssociationMenuItem() {
		if(newPlaceAssociationMenuItem == null){
			newPlaceAssociationMenuItem =
				new JMenuItem("Neu", KeyEvent.VK_N);
			
			InvObject object = (InvObject)node.getUserObject().getObject();
			
			if(existsPlaceObjectNotAt(object)){
				newPlaceAssociationMenuItem.addActionListener(
						new NewPlaceAssociationHandler(parent, object));
			}
			else {
				newPlaceAssociationMenuItem.setEnabled(false);
			}
		}
		return newPlaceAssociationMenuItem;
	}
	
	private JMenuItem getEditPlaceAssociationMenuItem() {
		if(editPlaceAssociationMenuItem == null){
			editPlaceAssociationMenuItem =
				new JMenuItem("Bearbeiten", KeyEvent.VK_B);
			
			InvObject object = (InvObject)node.getUserObject().getObject();
			
			if(hasPlaceAssociations(object)){
				editPlaceAssociationMenuItem.addActionListener(
						new EditPlaceAssociationHandler(parent, object));				
			}
			else {
				editPlaceAssociationMenuItem.setEnabled(false);
			}
		}
		return editPlaceAssociationMenuItem;
	}
	
	private JMenuItem getEditPlaceDataMenuItem(){
		if(editPlaceDataMenuItem == null){
			editPlaceDataMenuItem =
				new JMenuItem("Daten bearbeiten", KeyEvent.VK_D);
			
			Place place = (Place)node.getUserObject().getObject();
			editPlaceDataMenuItem.addActionListener(
					new EditPlaceHandler(parent, place));
		}
		return editPlaceDataMenuItem;
	}
	
	private JMenu getBeverageAssociationMenu() {
		if(beverageAssociationMenu == null){
			beverageAssociationMenu = new JMenu("Zuordnungen: Getrnke");
			beverageAssociationMenu.setMnemonic('G');
			
			if(existBeverages()){
				beverageAssociationMenu.add(getNewBeverageAssociationMenuItem());
				beverageAssociationMenu.add(getEditBeverageAssociationMenuItem());				
			}
			else {
				beverageAssociationMenu.setEnabled(false);
			}
		}
		return beverageAssociationMenu;
	}
	
	private JMenuItem getNewBeverageAssociationMenuItem() {
		if(newBeverageAssociationMenuItem == null){
			newBeverageAssociationMenuItem =
				new JMenuItem("Neu", KeyEvent.VK_N);
			
			Place place = (Place)node.getUserObject().getObject();
			
			if(existsBeverageNotAtPlace(place)){
				newBeverageAssociationMenuItem.addActionListener(
						new NewBeverageAssociationHandler(parent, place));
			}
			else {
				newBeverageAssociationMenuItem.setEnabled(false);
			}
		}
		return newBeverageAssociationMenuItem;
	}
	
	private JMenuItem getEditBeverageAssociationMenuItem() {
		if(editBeverageAssociationMenuItem == null){
			editBeverageAssociationMenuItem =
				new JMenuItem("Bearbeiten", KeyEvent.VK_B);
			
			Place place = (Place)node.getUserObject().getObject();
			
			if(hasBeverageAssociations(place)){
				editBeverageAssociationMenuItem.addActionListener(
						new EditBeverageAssociationHandler(parent, place));				
			}
			else {
				editBeverageAssociationMenuItem.setEnabled(false);
			}
		}
		return editBeverageAssociationMenuItem;
	}
	
	private JMenu getMiscAssociationMenu() {
		if(miscAssociationMenu == null){
			miscAssociationMenu = new JMenu("Zuordnungen: Sonstiges");
			miscAssociationMenu.setMnemonic('S');
			
			if(existMisc()){
				miscAssociationMenu.add(getNewMiscAssociationMenuItem());
				miscAssociationMenu.add(getEditMiscAssociationMenuItem());				
			}
			else {
				miscAssociationMenu.setEnabled(false);
			}
		}
		return miscAssociationMenu;
	}
	
	private JMenuItem getNewMiscAssociationMenuItem() {
		if(newMiscAssociationMenuItem == null){
			newMiscAssociationMenuItem =
				new JMenuItem("Neu", KeyEvent.VK_N);
			
			Place place = (Place)node.getUserObject().getObject();
			
			if(existsMiscNotAtPlace(place)){
				newMiscAssociationMenuItem.addActionListener(
						new NewMiscAssociationHandler(parent, place));
			}
			else {
				newMiscAssociationMenuItem.setEnabled(false);
			}
		}
		return newMiscAssociationMenuItem;
	}
	
	private JMenuItem getEditMiscAssociationMenuItem() {
		if(editMiscAssociationMenuItem == null){
			editMiscAssociationMenuItem =
				new JMenuItem("Bearbeiten", KeyEvent.VK_B);
			
			Place place = (Place)node.getUserObject().getObject();
			
			if(hasMiscAssociations(place)){
				editMiscAssociationMenuItem.addActionListener(
						new EditMiscAssociationHandler(parent, place));
			}
			else {
				editMiscAssociationMenuItem.setEnabled(false);
			}
		}
		return editMiscAssociationMenuItem;
	}
	
	private boolean hasPlaceAssociations(InvObject object){
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			return dbManager.hasPlaceAssociations(object);
		}
		catch(DBManagerLoadException e){
			new ExceptionHandler(parent).handle(e);
		}
		catch(SQLException e){
			new ExceptionHandler(parent).handle(e, sqlExceptionMsg);
		}
		
		return false;
	}
	
	private boolean hasBeverageAssociations(Place place){
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			return dbManager.hasBeverageAssociations(place);
		}
		catch(DBManagerLoadException e){
			new ExceptionHandler(parent).handle(e);
		}
		catch(SQLException e){
			new ExceptionHandler(parent).handle(e, sqlExceptionMsg);
		}
		
		return false;
	}
	
	private boolean hasMiscAssociations(Place place){
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			return dbManager.hasMiscAssociations(place);
		}
		catch(DBManagerLoadException e){
			new ExceptionHandler(parent).handle(e);
		}
		catch(SQLException e){
			new ExceptionHandler(parent).handle(e, sqlExceptionMsg);
		}
		
		return false;
	}
	
	private boolean existPlaces(){
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			return dbManager.existsPlace();
		}
		catch(DBManagerLoadException e){
			new ExceptionHandler(parent).handle(e);
		}
		catch(SQLException e){
			new ExceptionHandler(parent).handle(e, sqlExceptionMsg);
		}
		
		return false;
	}
	
	private boolean existBeverages(){
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			return dbManager.existsBeverage();
		}
		catch(DBManagerLoadException e){
			new ExceptionHandler(parent).handle(e);
		}
		catch(SQLException e){
			new ExceptionHandler(parent).handle(e, sqlExceptionMsg);
		}
		
		return false;
	}
	
	private boolean existMisc(){
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			return dbManager.existsMisc();
		}
		catch(DBManagerLoadException e){
			new ExceptionHandler(parent).handle(e);
		}
		catch(SQLException e){
			new ExceptionHandler(parent).handle(e, sqlExceptionMsg);
		}
		
		return false;
	}
	
	private boolean existsBeverageNotAtPlace(Place place){
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			return dbManager.existsBeverageNotAtPlace(place);
		}
		catch(DBManagerLoadException e){
			new ExceptionHandler(parent).handle(e);
		}
		catch(SQLException e){
			new ExceptionHandler(parent).handle(e, sqlExceptionMsg);
		}
		
		return false;
	}
	
	private boolean existsMiscNotAtPlace(Place place){
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			return dbManager.existsMiscNotAtPlace(place);
		}
		catch(DBManagerLoadException e){
			new ExceptionHandler(parent).handle(e);
		}
		catch(SQLException e){
			new ExceptionHandler(parent).handle(e, sqlExceptionMsg);
		}
		
		return false;
	}
	
	private boolean existsPlaceObjectNotAt(InvObject object){
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			return dbManager.existsPlaceObjectNotAt(object);
		}
		catch(DBManagerLoadException e){
			new ExceptionHandler(parent).handle(e);
		}
		catch(SQLException e){
			new ExceptionHandler(parent).handle(e, sqlExceptionMsg);
		}
		
		return false;
	}
}