package de.ba_leipzig.discoinv.db;

import java.io.File;
import java.sql.SQLException;
import java.util.*;

import de.matthiasrothe.util.collections.ArraySet;

public class DBManImpl implements DatabaseManager {
	private String file = "test.db";
	private int dbUpdateCount = 0;
	
	DBManImpl(){} // Verhindert Objekterzeugung auerhalb des db Packets
	
	public Set<Beverage> getBeverages(){
		ArraySet<Beverage> beverages = new ArraySet<Beverage>();
		Beverage[] drinks = new Beverage[5];
		
		for(int i = 0; i < drinks.length; i++){
			BeverageType type = new BeverageType(0, "Flasche");
			drinks[i] = new Beverage(i, "Getrnk " + i, false, false, 1000, type);
		}
		
		beverages.addAll(Arrays.asList(drinks));
		return beverages;
	}
	
	public Set<InvObject> getMisc(){
		ArraySet<InvObject> misc = new ArraySet<InvObject>();
		InvObject[] objects = new InvObject[4];
		
		for(int i = 0; i < objects.length; i++)
			objects[i] = new InvObject(i, "Sonstiges " + i);
		
		misc.addAll(Arrays.asList(objects));
		return misc;
	}
	
	public Set<Place> getPlaces(){
		ArraySet<Place> places = new ArraySet<Place>();
		Place[] placeArray = new Place[5];
		
		for(int i = 0; i < placeArray.length; i++){
			Place place = new Place(i, "Standort " + i);
			if(i > 0) place.setDepotID(0);
			placeArray[i] = place;
		}
		
		places.addAll(Arrays.asList(placeArray));
		return places;
	}
	
	private Set<AssociationInfo<InvObject>> getInvObjectInfo(){
		ArraySet<AssociationInfo<InvObject>> infoSet =
			new ArraySet<AssociationInfo<InvObject>>();
		Set<InvObject> objects = getMisc();
		objects.addAll(getBeverages());
		
		for(InvObject object : objects){
			AssociationInfo<InvObject> info = new AssociationInfo<InvObject>();
			
			info.setObject(object);
			info.setNeeded(2);
			info.setPresent(4);
			
			infoSet.add(info);
		}
		
		return infoSet;
	}
	
	public int getDBManagerID(){
		return file.hashCode();
	}
	
	public int getDBUpdateCount(){
		return dbUpdateCount;
	}

	public PlaceWithObjects getPlaceWithObjects(int id){
		PlaceWithObjects place = new PlaceWithObjects(-1, "Standort " + id);
		
		InvObject object = new InvObject(-1, "Sonstige 0");
		
		BeverageType type = new BeverageType(0, "Flasche");
		Beverage drink = 
			new Beverage(0, "Getrnk 0", false, false, 1000, type);
		
		BeverageWithLevel drinkWL =
			new BeverageWithLevel(0, "Getrnk mit Fllstand 0", false, false,
					1000, type, 10, 1010, 512);
		
		Set<AssociationInfo<InvObject>> associatedObjects =
			new ArraySet<AssociationInfo<InvObject>>();
		
		AssociationInfo<InvObject> associatedObject0 =
			new AssociationInfo<InvObject>();
		
		associatedObject0.setObject(object);
		associatedObject0.setNeeded(2);
		associatedObject0.setPresent(5);
		
		AssociationInfo<InvObject> associatedObject1 =
			new AssociationInfo<InvObject>();
		
		associatedObject1.setObject(drink);
		associatedObject1.setNeeded(1);
		associatedObject1.setPresent(3);
		
		AssociationInfo<InvObject> associatedObject2 =
			new AssociationInfo<InvObject>();
		
		associatedObject2.setObject(drinkWL);
		associatedObject2.setNeeded(3);
		associatedObject2.setPresent(4);
		
		associatedObjects.add(associatedObject0);
		associatedObjects.add(associatedObject1);
		associatedObjects.add(associatedObject2);
		
		place.setObjects(associatedObjects);
		
		return place;
	}

	public InvObject getInvObject(int id){
		InvObject object = new InvObject(0,"Test");
		return object;
	}

	public int getTotalPresent(InvObject object){
		return 0;
	}
	
	public int getTotalNeeded(InvObject object){
		return 0;
	}

	public Set<AssociationInfo<Place>> getAssociatedPlaces(
			InvObject invObject){
		ArraySet<AssociationInfo<Place>> associatedPlaces =
			new ArraySet<AssociationInfo<Place>>();
		
		for(int i = 0; i < 5; i++){
			AssociationInfo<Place> associatedPlace =
				new AssociationInfo<Place>();
			associatedPlace.setObject(new Place(i, "Standort " + i, 0));
			associatedPlace.setNeeded(2);
			associatedPlace.setPresent(3);
			associatedPlaces.add(associatedPlace);
		}
		
		return associatedPlaces;
	}

	public void closeDB() {
	}

	public boolean isOpenedDB() {
		return true;
	}

	public void openDB(File file) {
		this.file = file.getAbsolutePath();
	}

	public void saveDBAs(File file) {
	}

	public void addPlace(Place place) throws SQLException {
		System.out.println("Ein Standort wird hinzugefgt");
	}

	public Set<PlaceWithObjects> getPlacesWithObjects() throws SQLException {
		ArraySet<PlaceWithObjects> places = new ArraySet<PlaceWithObjects>();
		PlaceWithObjects[] placeArray = new PlaceWithObjects[5];
		
		for(int i = 0; i < placeArray.length; i++){
			PlaceWithObjects place = new PlaceWithObjects(i, "Standort " + i);
			if(i > 0) place.setDepotID(0);
			place.setObjects(getInvObjectInfo());
			placeArray[i] = place;
		}
		
		places.addAll(Arrays.asList(placeArray));
		return places;
	}

	public void addInvObject(InvObject object) throws SQLException {
		System.out.println("Ein Inventarobjekt wird hinzugefgt");
	}

	public Set<BeverageType> getBeverageTypes() throws SQLException {
		ArraySet<BeverageType> types = new ArraySet<BeverageType>();
		types.add(new BeverageType(0, "Flasche"));
		types.add(new BeverageType(1, "Fa"));
		return types;
	}

	public void addBeverage(Beverage beverage) throws SQLException {
		System.out.println("Getrnk (Typ: Beverage) wird hinzugefgt");
	}

	public void addBeverage(BeverageWithLevel beverage) throws SQLException {
		System.out.println("Getrnk (Typ: BeverageWithLevel) wird hinzugefgt");
	}

	public void newDB(File file) throws SQLException {
	}

	public void renameInvObject(int id, String newName) throws SQLException {
	}

	public void renamePlace(int id, String newName) throws SQLException {
	}

	public void deleteInvObject(int id) throws SQLException, DBIntegrityException {
	}

	public void deletePlace(int id) throws SQLException, DBIntegrityException {
	}

	public void updatePlace(Place place) throws SQLException {
	}

	public void updateBeverage(Beverage beverage) throws SQLException {
	}

	public void updateBeverage(BeverageWithLevel beverage) throws SQLException {
	}

	public Set<Beverage> getBeveragesNotAtPlace(Place place) throws SQLException {
		return null;
	}

	public Set<InvObject> getMiscNotAtPlace(Place place) throws SQLException {
		return null;
	}

	public Set<Place> getPlacesObjectNotAt(InvObject object) throws SQLException {
		return null;
	}

	public boolean existsBeverage() throws SQLException {
		return true;
	}

	public boolean existsMisc() throws SQLException {
		return true;
	}

	public boolean existsPlace() throws SQLException {
		return true;
	}

	public boolean existsBeverageNotAtPlace(Place place) throws SQLException {
		return true;
	}

	public boolean existsMiscNotAtPlace(Place place) throws SQLException {
		return true;
	}

	public boolean existsPlaceObjectNotAt(InvObject object) throws SQLException {
		return true;
	}

	public boolean hasBeverageAssociations(Place place) throws SQLException {
		return true;
	}

	public boolean hasMiscAssociations(Place place) throws SQLException {
		return true;
	}

	public boolean hasPlaceAssociations(InvObject object) throws SQLException {
		return true;
	}

	public void addObjectAssociations(Place place,
			Set<? extends AssociationInfo<? extends Identifiable>> objectInfos)
	throws SQLException {
	}

	public void addPlaceAssociations(InvObject object,
			Set<? extends AssociationInfo<? extends Identifiable>> objectInfos)
	throws SQLException {
	}

	public void updateObjectAssociations(Place place,
			Set<? extends AssociationInfo<? extends Identifiable>> objectInfos)
	throws SQLException {
	}

	public void updatePlaceAssociations(InvObject object,
			Set<? extends AssociationInfo<? extends Identifiable>> objectInfos)
	throws SQLException {
	}

	public Set<AssociationInfo<? extends InvObject>> getAssociatedObjects(Place place) throws SQLException {
		// TODO Auto-generated method stub
		return null;
	}
}