package de.ba_leipzig.discoinv.db;

import java.io.*;
import java.sql.*;
import java.util.Set;

import de.matthiasrothe.util.collections.ArraySet;

public class SQLiteDBManager implements DatabaseManager {
	
	private Connection conn = null;
	private int updatecount = 0;
	private String fileName;
	private boolean del = true;
	
	SQLiteDBManager() throws ClassNotFoundException {
		//	SQLite JDBC treiber laden
		Class.forName("org.sqlite.JDBC");
		
		System.out.println("JDBC geladen.");
	} // Verhindert Objekterzeugung auerhalb des db Packets
	
	public Set<Beverage> getBeverages() throws SQLException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage ausfhren
		String query = "SELECT * FROM Drinks";
		ResultSet rs = stmt.executeQuery( query );
		
		// InvObjecte erzeugen
		Set <Beverage> objects = new ArraySet<Beverage>();
		while ( rs.next() ) {
			InvObject o=getInvObjectForResultSet(rs);
			if(o instanceof Beverage) objects.add((Beverage) o);
		}
		
		// Freigabe
		rs.close();
		stmt.close();

		return objects;
		
	}

	public int getDBManagerID() {
		return fileName.hashCode();
	}

	public int getDBUpdateCount() {
		return updatecount;
	}

	public InvObject getInvObject(int id) throws SQLException{
		InvObject o=null;
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage ausfhren
		String query = "SELECT * FROM Objects WHERE id="+id;
		ResultSet rs = stmt.executeQuery( query );
		
		// Prfen ob gefunden
		if(rs.next())
			o = getInvObjectForResultSet(rs);

		// Freigabe
		rs.close();
		stmt.close();

		return o;
	}
	
	/**
	 * Erzeugt fr die aktuelle Zeile des ResultSet ein Inventar-Object
	 * @param rs
	 * @return InvObject or Beverage or BeverageWithLevel
	 * @throws SQLException
	 */
	private InvObject getInvObjectForResultSet(ResultSet rs) throws SQLException {
		InvObject o = null;
		
		// InvObject erzeugen
		int oid = rs.getInt(1);
		String name = rs.getString(2);
		boolean alc = rs.getBoolean(3);
		if(rs.wasNull()) {	// Es ist ein Normales Object
			o = new InvObject(oid,name);
		} else {
			boolean refund=rs.getBoolean(4);
			int cap = rs.getInt(5);
			BeverageType type = new BeverageType(rs.getInt(7),rs.getString(6));
			int empty = rs.getInt(8);
			
			if(rs.wasNull()) { // es ist ein geschlossenes Getrnk
				o = new Beverage(oid, name, alc, refund, cap, type);
			} else { // es ist ein offenes Getrnk
				int fill=rs.getInt(9);
				o = new BeverageWithLevel(oid, name, alc, refund, cap, type, empty, fill);
			}
		}
		
		return o;
	}

	public Set<InvObject> getMisc() throws SQLException{
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage ausfhren
		String query = "SELECT * FROM NonDrinks";
		ResultSet rs = stmt.executeQuery( query );
		
		// InvObjecte erzeugen
		Set <InvObject> objects = new ArraySet<InvObject>();
		while ( rs.next() ) {
			InvObject o=new InvObject(rs.getInt(1),rs.getString(2));
			objects.add(o);
		}
		
		// Freigabe
		rs.close();
		stmt.close();

		return objects;
	}

	private Place getPlace(int id) throws SQLException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage ausfhren
		String query = "SELECT * FROM place WHERE id="+id;
		ResultSet rs = stmt.executeQuery( query );
		
		// Prfen ob gefunden
		if(!rs.next()) return null;
		
		// Place-Object erzeugen
		Place p=new Place(rs.getInt(1),rs.getString(2));
		int depoID=rs.getInt(3);
		if(!rs.wasNull()) p.setDepotID(depoID);
		
		// Freigabe
		rs.close();
		stmt.close();
		
		return p;
	}
	
	public PlaceWithObjects getPlaceWithObjects(int id) throws SQLException {
		Place place = getPlace(id);
		PlaceWithObjects pwo=new PlaceWithObjects(place);
		pwo.setObjects(getAssociatedObjects(place));
		return pwo;
	}

	public Set<Place> getPlaces() throws SQLException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage ausfhren
		String query = "SELECT * FROM place";
		ResultSet rs = stmt.executeQuery( query );
		
		// Place-Objecte erzeugen
		Set <Place> places = new ArraySet<Place>();
		while ( rs.next() ) {
			// Place erzeugen
			Place p=new Place(rs.getInt(1),rs.getString(2));
			int depoID=rs.getInt(3);
			if(!rs.wasNull()) p.setDepotID(depoID);
			
			// Place dem array hinzufgen
			places.add(p);
		}
		
		// Freigabe
		rs.close();
		stmt.close();

		return places;
	}

	public Set<PlaceWithObjects> getPlacesWithObjects() throws SQLException {
		Set<Place> places = getPlaces();
		Set<PlaceWithObjects> pwos=new ArraySet<PlaceWithObjects>();
		
		for(Place p : places) {
			PlaceWithObjects pwo = new PlaceWithObjects(p);
			// Objecte fr den Standort
			pwo.setObjects(getAssociatedObjects(p));
			// zum array hinzufgen
			pwos.add(pwo);
		}

		return pwos;
	}
	
	public Set<AssociationInfo<InvObject>> getAssociatedObjects(Place place)
	throws SQLException {
		int id = place.getID();
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage zusammensetzen
		String query = "SELECT object, SUM(has), SUM(need) FROM (\n";
		query += "SELECT object, c AS 'has', 0 AS 'need' FROM has WHERE place="+id+"\n";
		query += "UNION SELECT object, 0 AS 'has', c AS 'need' FROM needmin WHERE place="+id+"\n";
		query += ") GROUP BY object";
		
		// Abrage ausfhren
		ResultSet rs = stmt.executeQuery(query);
		
		// Set zusammensetzen
		Set<AssociationInfo<InvObject>> objects =
			new ArraySet<AssociationInfo<InvObject>>();
		
		while(rs.next()) {
			AssociationInfo<InvObject> ai =
				new AssociationInfo<InvObject>(getInvObject(rs.getInt(1)), rs.getInt(2), rs.getInt(3));
			objects.add(ai);
		}
		
		// Freigabe
		rs.close();
		stmt.close();
		
		return objects;
	}

	public Set<AssociationInfo<Place>> getAssociatedPlaces(
			InvObject invObject) throws SQLException {
		
		int id = invObject.getID();
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage zusammensetzen
		String query = "SELECT place, SUM(has), SUM(need) FROM (\n";
		query += "SELECT place, c AS 'has', 0 AS 'need' FROM has WHERE object="+id+"\n";
		query += "UNION SELECT place, 0 AS 'has', c AS 'need' FROM needmin WHERE object="+id+"\n";
		query += ") GROUP BY place";
		
		// Abrage ausfhren
		ResultSet rs = stmt.executeQuery(query);
		
		// Set zusammensetzen
		Set<AssociationInfo<Place>> associatedPlaces =
			new ArraySet<AssociationInfo<Place>>();
		
		while(rs.next()) {
			AssociationInfo<Place> associatedPlace =
				new AssociationInfo<Place>(getPlace(rs.getInt(1)), rs.getInt(2), rs.getInt(3));
			associatedPlaces.add(associatedPlace);
		}
		
		// Freigabe
		rs.close();
		stmt.close();
		
		return associatedPlaces;
	}
	
	public int getTotalNeeded(InvObject object) throws SQLException{
		int count=0;
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage ausfhren
		String query = "SELECT SUM(c) FROM needmin WHERE object="+object.getID();
		ResultSet rs = stmt.executeQuery( query );
		
		// Wert holen
		if(rs.next()) count=rs.getInt(1);
		
		// Freigabe
		rs.close();
		stmt.close();
		
		return count;
	}

	public int getTotalPresent(InvObject object) throws SQLException{
		int count=0;
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage ausfhren
		String query = "SELECT SUM(c) FROM has WHERE object="+object.getID();
		ResultSet rs = stmt.executeQuery( query );
		
		// Wert holen
		if(rs.next()) count=rs.getInt(1);
		
		// Freigabe
		rs.close();
		stmt.close();
		
		return count;
	}

	public void closeDB() throws SQLException {
		if ( conn != null ) {
    		conn.close();
    		conn = null;
		}
	}

	public boolean isOpenedDB() {
		return (conn!=null);
	}

	/**
	 * Kopiert die Dateien, wird fr SaveAs bentigt
	 * @param src
	 * @param dest
	 */
	private void copyFile( String src, String dest ){ 
		FileInputStream  fis = null; 
		FileOutputStream fos = null; 

		try 
		{ 
			fis = new FileInputStream( src ); 
			fos = new FileOutputStream( dest ); 

			byte[] buffer = new byte[ 0xFFFF ]; 

			for ( int len; (len = fis.read(buffer)) != -1; ) 
				fos.write( buffer, 0, len );
		} 
		catch ( IOException e ) { } 
		finally { 
			if ( fis != null ) 
				try { fis.close(); } catch ( IOException e ) { } 
				if ( fos != null ) 
					try { fos.close(); } catch ( IOException e ) { } 
		} 
	} 
	
	
	public void openDB(File file) throws SQLException, DBIntegrityException{
		// vorsichtshalber
		closeDB();
		
		// zur DB verbinden
		conn = DriverManager.getConnection("jdbc:sqlite:"+file.getAbsolutePath());
		fileName = file.getAbsolutePath();
		
		// TODO Verbessern: DB-Datei auf Korrektheit prfen
		DatabaseMetaData meta = conn.getMetaData();
		String [] types = {"TABLE", "VIEW"};
		ResultSet rs = meta.getTables((String) null, (String) null, "%", types );
		int count=0;
		while(rs.next()) count++;
		
		// 8 Tabellen + 3 Views + 1 Systemtabelle + (1 SQLiteAdmin-Tabelle)
		if(count < 12 || count > 13) throw new DBIntegrityException("Keine gltige Datenbank dieses Programmes!");
		
		System.out.println("Verbindung zur DB hergestellt.");
	}

	
	public void saveDBAs(File file) {
		// aktuelle DB Kopieren
		copyFile(fileName, file.getAbsolutePath());
		
		// neue Datei ffnen
		try {
			openDB(file);
		} catch (DBIntegrityException e) {} catch (SQLException e) {}
	}

	public void addPlace(Place place) throws SQLException, DBIntegrityException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// testen ob es schon existiert
		ResultSet rs = stmt.executeQuery("SELECT * FROM place WHERE bez LIKE '"+place.getName()+"'");
		if(rs.next()) throw new DBIntegrityException("Ein Ort mit diesem Namen existiert bereits.");
		
		// Freigabe
		rs.close();
		stmt.close();
		
		// einfgen
		updatePlace(place);
	}

	public void addInvObject(InvObject object) throws SQLException, DBIntegrityException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// testen ob es schon existiert
		ResultSet rs = stmt.executeQuery("SELECT * FROM NonDrinks WHERE bez LIKE '"+object.getName()+"'");
		if(rs.next()) throw new DBIntegrityException("Ein Object mit diesem Namen existiert bereits.");

		// einfgen
		addInvObject(stmt, object);
		
		// Freigabe
		rs.close();
		stmt.close();
	}
	
	/**
	 * Fgt ein InventarObject in die Datenbank ein 
	 * @param stmt
	 * @param object
	 * @return die ID des eingefgten Objectes
	 * @throws SQLException
	 */
	private int addInvObject(Statement stmt, InvObject object) throws SQLException {
		int id=0;
		String query="REPLACE INTO object VALUES (";
		query+=((object.getID()==0)?("NULL"):(object.getID()));
		query+=",'"+object.getName()+"')";
		
		// einfgen
		if(stmt.executeUpdate(query)!=0) updatecount++;	
		
		// eingefgte id holen
		if(object.getID() == 0){
			id=getLastInsertID(stmt);
		}
		else {
			id = object.getID();
		}
		
		return id;
	}

	public Set<BeverageType> getBeverageTypes() throws SQLException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage ausfhren
		String query = "SELECT * FROM objecttype";
		ResultSet rs = stmt.executeQuery( query );
		
		// Place-Objecte erzeugen
		Set <BeverageType> types = new ArraySet<BeverageType>();
		while ( rs.next() ) {
			// Place erzeugen
			BeverageType t=new BeverageType(rs.getInt(1),rs.getString(2));
			
			// Place dem array hinzufgen
			types.add(t);
		}
		
		// Freigabe
		rs.close();
		stmt.close();

		return types;
	}

	public void addBeverage(Beverage beverage) throws SQLException, DBIntegrityException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// testen ob es schon existiert	
		ResultSet rs = stmt.executeQuery("SELECT * FROM Drinks WHERE bez LIKE '"+beverage.getName()+"'");
		if(rs.next()) throw new DBIntegrityException("Ein Objekt mit diesem Namen existiert bereits.");
		
		// Freigabe
		rs.close();
		stmt.close();
		
		// einfgen
		updateBeverage(beverage);
	}

	public void addBeverage(BeverageWithLevel beverage) throws SQLException, DBIntegrityException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// testen ob es schon existiert
		ResultSet rs = stmt.executeQuery("SELECT * FROM Drinks WHERE bez LIKE '"+beverage.getName()+"'");
		if(rs.next()) throw new DBIntegrityException("Ein Objekt mit diesem Namen existiert bereits.");
		
		// Freigabe
		rs.close();
		stmt.close();
		
		// einfgen
		updateBeverage(beverage);
	}

	public void newDB(File file) throws SQLException {
		// vorsichtshalber
		closeDB();
		
		// zur DB verbinden
		conn = DriverManager.getConnection("jdbc:sqlite:"+file.getAbsolutePath());
		fileName = file.getAbsolutePath();
		updatecount=0;
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// DB struktur anlegen
		conn.setAutoCommit(false);
		try {
			
			stmt.addBatch("CREATE TABLE [drink] ("+
					"[id] INTEGER  NULL PRIMARY KEY,"+
					"[alc] BOOLEAN  NOT NULL,"+
					"[refund] BOOLEAN  NOT NULL,"+
					"[capacity] INTEGER  NOT NULL,"+
					"[type] INTEGER  NOT NULL)");
			
			stmt.addBatch("CREATE TABLE [has] ("+
					"[place] INTEGER  NOT NULL,"+
					"[object] INTEGER  NOT NULL,"+
					"[c] INTEGER  NOT NULL,"+
					"PRIMARY KEY ([place],[object]))");
			
			stmt.addBatch("CREATE TABLE [needmin] ("+
					"[place] INTEGER  NOT NULL,"+
					"[object] INTEGER  NOT NULL,"+
					"[c] INTEGER  NOT NULL,"+
					"PRIMARY KEY ([place],[object]))");
			
			stmt.addBatch("CREATE TABLE [haswl] ("+
					"[id] INTEGER  NOT NULL PRIMARY KEY AUTOINCREMENT,"+
					"[place] INTEGER  NOT NULL,"+
					"[object] INTEGER  NOT NULL,"+
					"[level] INTEGER  NOT NULL)");
			
			stmt.addBatch("CREATE TABLE [object] ("+
					"[id] INTEGER  PRIMARY KEY AUTOINCREMENT NOT NULL,"+
					"[bez] VARCHAR(60)  NOT NULL)");
			
			stmt.addBatch("CREATE TABLE [objecttype] ("+
					"[id] INTEGER  PRIMARY KEY AUTOINCREMENT NOT NULL,"+
					"[bez] VARCHAR(15)  NOT NULL)");
			
			stmt.addBatch("CREATE TABLE [place] ("+
					"[id] INTEGER  PRIMARY KEY AUTOINCREMENT NOT NULL,"+
					"[bez] VARCHAR(50)  NOT NULL,"+
					"[depotid] INTEGER  NULL)");
			
			stmt.addBatch("CREATE TABLE [withlevel] ("+
					"[id] INTEGER  NULL PRIMARY KEY,"+
					"[empty] INTEGER  NOT NULL,"+
					"[filled] INTEGER  NOT NULL)");
			
			stmt.addBatch("CREATE VIEW [Objects] AS "+
					"SELECT object.id AS 'id', object.bez AS 'bez',"+
					"       drink.alc AS 'alc', drink.refund AS 'refund',"+
					"       drink.capacity AS 'capacity',"+
					"       objecttype.bez AS 'type',"+
					"       objecttype.id as 'typeid',"+
					"       withlevel.empty AS 'empty',"+
					"       withlevel.filled AS 'filled' "+
					"FROM object LEFT JOIN drink ON object.id=drink.id"+
					"            LEFT JOIN withlevel ON drink.id=withlevel.id"+
					"            LEFT JOIN objecttype ON drink.'type' = objecttype.id "+
					"ORDER BY object.bez;");
			
			stmt.addBatch("CREATE VIEW [Drinks] AS "+
					"SELECT * FROM Objects WHERE capacity IS NOT NULL;");
			
			stmt.addBatch("CREATE VIEW [NonDrinks] AS "+
					"SELECT * FROM Objects WHERE capacity IS NULL;");
			
			stmt.addBatch("CREATE TRIGGER [Trigger_Delete_Object]\n"+ 
					"AFTER DELETE ON [object]\n"+ 
					"FOR EACH ROW \n"+
					"BEGIN \n"+
					"DELETE FROM drink WHERE id=OLD.id;\n"+
					"DELETE FROM withlevel WHERE id=OLD.id;\n"+
					"DELETE FROM haswl WHERE object=OLD.id;\n"+
					"DELETE FROM has WHERE object=Old.id;\n"+
					"DELETE FROM needmin WHERE object=OLD.id;\n"+
					"END");
			
			stmt.addBatch("CREATE TRIGGER [Trigger_Delete_Place]\n"+ 
					"AFTER DELETE ON [place]\n"+ 
					"FOR EACH ROW \n"+
					"BEGIN \n"+
					"DELETE FROM haswl WHERE place=OLD.id;\n"+
					"DELETE FROM has WHERE place=OLD.id;\n"+
					"DELETE FROM needmin WHERE place=OLD.id;\n"+
					"END");
			
			stmt.addBatch("INSERT INTO objecttype VALUES (1,'Flasche')");
			stmt.addBatch("INSERT INTO objecttype VALUES (2,'Fass/Puck')");
			stmt.addBatch("INSERT INTO objecttype VALUES (3,'Dose')");
			stmt.addBatch("INSERT INTO objecttype VALUES (4,'Tetrapack')");
			
			stmt.executeBatch();
		}
		catch ( BatchUpdateException e ) {
			conn.setAutoCommit(true);
			conn.rollback();
		}
		
		// freigabe
		stmt.close();
		conn.setAutoCommit(true);
	}

	public void renameInvObject(int id, String newName) throws SQLException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage erstellen
		String query="UPDATE object SET bez='"+newName+"' WHERE id="+id;
		
		// Action
		if(stmt.executeUpdate(query)!=0) updatecount++;
		
		// freigabe
		stmt.close();
	}

	
	public void renamePlace(int id, String newName) throws SQLException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage erstellen
		String query="UPDATE place SET bez='"+newName+"' WHERE id="+id;
		
		// Action
		if(stmt.executeUpdate(query)!=0) updatecount++;
		
		// freigabe
		stmt.close();
	}

	public void deleteInvObject(int id) throws SQLException, DBIntegrityException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Prfen ob das Object noch irgendwo gebraucht wird
		ResultSet rs=stmt.executeQuery("SELECT place FROM needmin WHERE object="+id);
		if(rs.next())  throw new DBIntegrityException("Das Object ist noch als Bedarf zugeorndet.");
		rs.close();
		
		// Prfen ob das Object noch irgendwo zugeordnet ist
		rs=stmt.executeQuery("SELECT place FROM has WHERE object="+id);
		if(rs.next())  throw new DBIntegrityException("Das Object ist noch einem Ort zugeorndet.");
		rs.close();
		rs=stmt.executeQuery("SELECT place FROM haswl WHERE object="+id);
		if(rs.next())  throw new DBIntegrityException("Das Object ist noch einem Ort zugeorndet.");
		rs.close();
		
		// entfernen
		// ein Trigger entfernt weitere Eigenschaften aus den Tabellen
		// 'drink' und 'withlevel'
		if(stmt.executeUpdate("DELETE FROM object WHERE id="+id)!=0) updatecount++;
		
	}

	public void deletePlace(int id) throws SQLException, DBIntegrityException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// prfen ob es als Lager fungiert
		ResultSet rs=stmt.executeQuery("SELECT id FROM place WHERE depotid="+id);
		if(rs.next()) throw new DBIntegrityException("Der Ort ist als Lager bei mindestens einem Ort eingetragen.");
		rs.close();
		
		// entfernen
		// ACHTUNG: ein Trigger entfernt alle referenzen auf den Ort
		// also alle Eintrge in den Tabellen 'has', 'haswl' und 'needmin'
		if(stmt.executeUpdate("DELETE FROM place WHERE id="+id)!=0) updatecount++;
		
		// Freigabe
		stmt.close();
	}
	
	
	private int getLastInsertID(Statement stmt) throws SQLException {
		int id=0;
		ResultSet rs = stmt.executeQuery("SELECT last_insert_rowid()");
		
		rs.next();
		id=rs.getInt(1);
		
		// Freigabe
		rs.close();
		
		return id;
	}

	
	public void updatePlace(Place place) throws SQLException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Query zusammensetzen
		String query="REPLACE INTO place VALUES (";
		query+=((place.getID()==0)?("NULL"):(place.getID()));
		query+=",'"+place.getName()+"',";
		if(place.getDepotID()==null) query+="NULL";
		else query+=place.getDepotID();
		query+=")";
			
		// einfgen
		if(stmt.executeUpdate(query)!=0) updatecount++;
		
		// Freigabe
		stmt.close();
	}

	
	public void updateBeverage(Beverage beverage) throws SQLException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// einfgen in object
		int id=addInvObject(stmt, beverage);
		
		// Query fr das einfgen in drink
		String query="REPLACE INTO drink VALUES ";
		query+="("+id+","+(beverage.isAlc()?(1):(0))+","+(beverage.isRefund()?(1):(0));
		query+=","+beverage.getCapacity()+","+beverage.getType().getId()+")";
		
		// einfgen
		if(stmt.executeUpdate(query)!=0) updatecount++;	
		
		// entfernen aus withlevel
		stmt.executeUpdate("DELETE FROM withlevel WHERE id="+id);
		
		// Freigabe
		stmt.close();

	}

	public void updateBeverage(BeverageWithLevel beverage) throws SQLException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// einfgen in object
		int id=addInvObject(stmt, beverage);
		
		// Query fr das einfgen in drink
		String query="REPLACE INTO drink VALUES ";
		query+="("+id+","+(beverage.isAlc()?(1):(0))+","+(beverage.isRefund()?(1):(0));
		query+=","+beverage.getCapacity()+","+beverage.getType().getId()+")";
		
		// einfgen
		if(stmt.executeUpdate(query)!=0) {
			updatecount++;	
			
			// Query fr einfgen in withlevel
			query="REPLACE INTO withlevel VALUES ";
			query+="("+id+","+beverage.getEmpty()+","+beverage.getFull()+")";
			
			// einfgen in withlevel
			if(stmt.executeUpdate(query)!=0) updatecount++;;
		}
		
		// Freigabe
		stmt.close();

	}

	public Set<Beverage> getBeveragesNotAtPlace(Place place) throws SQLException {
		int id = place.getID();
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage zusammensetzen
		String query = "SELECT id, alc, present FROM Objects LEFT JOIN (\n";
		query +="SELECT object, 1 AS 'present' FROM has WHERE place="+id+"\n";
		query +="UNION SELECT object, 1 AS 'present' FROM needmin WHERE place="+id+")\n";
		query +="ON id=object WHERE present IS NULL AND alc IS NOT NULL ORDER BY id";
		
		// Abrage ausfhren
		ResultSet rs = stmt.executeQuery(query);
		
		// ergebniss auswerten
		Set <Beverage> sb = new ArraySet<Beverage>();
		while ( rs.next() ) {
			// Place erzeugen
			Beverage b=(Beverage) getInvObject(rs.getInt(1));
			
			// dem array hinzufgen
			sb.add(b);
		}
		
		// Freigabe
		rs.close();
		stmt.close();
		
		// ergebniss zurckgeben
		return sb;
	}

	public Set<InvObject> getMiscNotAtPlace(Place place) throws SQLException {
		int id = place.getID();
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage zusammensetzen
		String query = "SELECT id, alc, present FROM Objects LEFT JOIN (\n";
		query +="SELECT object, 1 AS 'present' FROM has WHERE place="+id+"\n";
		query +="UNION SELECT object, 1 AS 'present' FROM needmin WHERE place="+id+")\n";
		query +="ON id=object WHERE present IS NULL AND alc IS NULL ORDER BY id";
		
		// Abrage ausfhren
		ResultSet rs = stmt.executeQuery(query);
		
		// ergebniss auswerten
		Set <InvObject> objects = new ArraySet<InvObject>();
		while ( rs.next() ) {
			// Place erzeugen
			InvObject o=getInvObject(rs.getInt(1));
			
			// dem array hinzufgen
			objects.add(o);
		}
		
		// Freigabe
		rs.close();
		stmt.close();
		
		// ergebniss zurckgeben
		return objects;
	}

	public Set<Place> getPlacesObjectNotAt(InvObject object) throws SQLException {
		int id = object.getID();
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage zusammensetzen
		String query = "SELECT id, present FROM place LEFT JOIN (\n";
		query +="SELECT place, 1 AS 'present' FROM has WHERE object="+id+"\n";
		query +="UNION SELECT place, 1 AS 'present' FROM needmin WHERE object="+id+")\n";
		query +="ON id=place WHERE present IS NULL ORDER BY id";
		
		// Abrage ausfhren
		ResultSet rs = stmt.executeQuery(query);
		
		// ergebniss auswerten
		Set <Place> places = new ArraySet<Place>();
		while ( rs.next() ) {
			// Place erzeugen
			Place p=getPlace(rs.getInt(1));
			
			// dem array hinzufgen
			places.add(p);
		} 
		
		// Freigabe
		rs.close();
		stmt.close();
		
		return places;
	}

	public boolean existsBeverage() throws SQLException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// alle beverages abfragen
		ResultSet rs=stmt.executeQuery("SELECT * FROM Drinks");
		boolean exist=rs.next();
		
		// Freigabe
		rs.close();
		stmt.close();
		
		// Ergebniss zurckgeben
		return exist;
	}

	public boolean existsMisc() throws SQLException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// alle miscs abfragen
		ResultSet rs=stmt.executeQuery("SELECT * FROM NonDrinks");
		boolean exist=rs.next();
		
		// Freigabe
		rs.close();
		stmt.close();
		
		// Ergebniss zurckgeben
		return exist;
	}

	public boolean existsPlace() throws SQLException {
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// alle places abfragen
		ResultSet rs=stmt.executeQuery("SELECT * FROM place");
		boolean exist=rs.next();
		
		// Freigabe
		rs.close();
		stmt.close();
		
		// Ergebniss zurckgeben
		return exist;
	}

	public boolean existsBeverageNotAtPlace(Place place) throws SQLException {
		int id = place.getID();
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage zusammensetzen
		String query = "SELECT id, alc, present FROM Objects LEFT JOIN (\n";
		query +="SELECT object, 1 AS 'present' FROM has WHERE place="+id+"\n";
		query +="UNION SELECT object, 1 AS 'present' FROM needmin WHERE place="+id+")\n";
		query +="ON id=object WHERE present IS NULL AND alc IS NOT NULL ORDER BY id";
		
		// Abrage ausfhren
		ResultSet rs = stmt.executeQuery(query);
		
		// was gefunden?
		boolean exist=rs.next(); 
		
		// Freigabe
		rs.close();
		stmt.close();
		
		return exist;
	}

	public boolean existsMiscNotAtPlace(Place place) throws SQLException {
		int id = place.getID();
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage zusammensetzen
		String query = "SELECT id, alc, present FROM Objects LEFT JOIN (\n";
		query +="SELECT object, 1 AS 'present' FROM has WHERE place="+id+"\n";
		query +="UNION SELECT object, 1 AS 'present' FROM needmin WHERE place="+id+")\n";
		query +="ON id=object WHERE present IS NULL AND alc IS NULL ORDER BY id";
		
		// Abrage ausfhren
		ResultSet rs = stmt.executeQuery(query);
		
		// was gefunden?
		boolean exist=rs.next(); 
		
		// Freigabe
		rs.close();
		stmt.close();
		
		return exist;
	}

	public boolean existsPlaceObjectNotAt(InvObject object) throws SQLException {
		int id = object.getID();
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage zusammensetzen
		String query = "SELECT id, present FROM place LEFT JOIN (\n";
		query +="SELECT place, 1 AS 'present' FROM has WHERE object="+id+"\n";
		query +="UNION SELECT place, 1 AS 'present' FROM needmin WHERE object="+id+")\n";
		query +="ON id=place WHERE present IS NULL";
		
		// Abrage ausfhren
		ResultSet rs = stmt.executeQuery(query);
		
		// was gefunden?
		boolean exist=rs.next(); 
		
		// Freigabe
		rs.close();
		stmt.close();
		
		return exist;
	}

	public boolean hasBeverageAssociations(Place place) throws SQLException {
		int id = place.getID();
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage zusammensetzen
		String query = "SELECT id, alc, present FROM Objects LEFT JOIN (\n";
		query +="SELECT object, 1 AS 'present' FROM has WHERE place="+id+"\n";
		query +="UNION SELECT object, 1 AS 'present' FROM needmin WHERE place="+id+")\n";
		query +="ON id=object WHERE present IS NOT NULL AND alc IS NOT NULL";
		
		// Abrage ausfhren
		ResultSet rs = stmt.executeQuery(query);
		
		// was gefunden?
		boolean exist=rs.next(); 
		
		// Freigabe
		rs.close();
		stmt.close();
		
		return exist;
	}

	public boolean hasMiscAssociations(Place place) throws SQLException {
		int id = place.getID();
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage zusammensetzen
		String query = "SELECT id, alc, present FROM Objects LEFT JOIN (\n";
		query +="SELECT object, 1 AS 'present' FROM has WHERE place="+id+"\n";
		query +="UNION SELECT object, 1 AS 'present' FROM needmin WHERE place="+id+")\n";
		query +="ON id=object WHERE present IS NOT NULL AND alc IS NULL";
		
		// Abrage ausfhren
		ResultSet rs = stmt.executeQuery(query);
		
		// was gefunden?
		boolean exist=rs.next(); 
		
		// Freigabe
		rs.close();
		stmt.close();
		
		return exist;
	}

	public boolean hasPlaceAssociations(InvObject object) throws SQLException {
		int id = object.getID();
		
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		
		// Abfrage zusammensetzen
		String query = "SELECT id, present FROM place LEFT JOIN (\n";
		query +="SELECT place, 1 AS 'present' FROM has WHERE object="+id+"\n";
		query +="UNION SELECT place, 1 AS 'present' FROM needmin WHERE object="+id+")\n";
		query +="ON id=place WHERE present IS NOT NULL";
		
		// Abrage ausfhren
		ResultSet rs = stmt.executeQuery(query);
		
		// was gefunden?
		boolean exist=rs.next(); 
		
		// Freigabe
		rs.close();
		stmt.close();
		
		return exist;
	}

	
	public void addObjectAssociations(Place place,
			Set<? extends AssociationInfo<? extends Identifiable>> associations)
	throws SQLException {
		// ich wei nicht sehr Elegant
		// aber da ich hier nicht alle Objecte bekomme, kann ich auch nicht alle lsche!
		del=false;
		updateObjectAssociations(place, associations);
		del=true;
	}

	public void addPlaceAssociations(InvObject object,
			Set<? extends AssociationInfo<? extends Identifiable>> associations)
	throws SQLException {
		del=false;
		updatePlaceAssociations(object, associations);
		del=true;
	}

	public void updateObjectAssociations(Place place,
			Set<? extends AssociationInfo<? extends Identifiable>> associations)
	throws SQLException {
		// TODO Objecte einem Ort zuornden
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		int pid=place.getID();
		
		// nderungen in einem Rutsch
		conn.setAutoCommit(false);
		try {
			// Batch-Abfrage zusammensetzen
			// vorhande erst mal entfernen  (nur bei Update nicht bei add)
			if(del) {
				stmt.addBatch("DELETE FROM has WHERE place="+pid);
				stmt.addBatch("DELETE FROM needmin WHERE place="+pid);
			}
			
			// Eintrge Aufnehmen
			for(AssociationInfo ai : associations) {
				// in 'has'
				stmt.addBatch("INSERT INTO has VALUES("+pid+","+ai.getObject().getID()+","+ai.getPresent()+")");
				// und in 'needmin'
				stmt.addBatch("INSERT INTO needmin VALUES("+pid+","+ai.getObject().getID()+","+ai.getNeeded()+")");
			}
			
			// Abfrage ausfhren
			stmt.executeBatch();
		}
		catch ( BatchUpdateException e ) {
			conn.setAutoCommit(true);
			conn.rollback();
		}
		
		conn.setAutoCommit(true);
		updatecount++;
		
		// freigabe
		stmt.close();
	}

	public void updatePlaceAssociations(InvObject object,
			Set<? extends AssociationInfo<? extends Identifiable>> associations)
	throws SQLException {
		// TODO Orte einem Object zuordnen
		// Statement vorbereiten
		Statement stmt = conn.createStatement();
		int oid=object.getID();
		
		// nderungen in einem Rutsch
		conn.setAutoCommit(false);
		try {
			// Batch-Abfrage zusammensetzen
			
			// vorhande erst mal entfernen (nur bei Update nicht bei add)
			if(del) {
				stmt.addBatch("DELETE FROM has WHERE object="+oid);
				stmt.addBatch("DELETE FROM needmin WHERE object="+oid);
			}
			
			// Eintrge Aufnehmen
			for(AssociationInfo ai : associations) {
				// in 'has'
				stmt.addBatch("INSERT INTO has VALUES("+ai.getObject().getID()+","+oid+","+ai.getPresent()+")");
				// und in 'needmin'
				stmt.addBatch("INSERT INTO needmin VALUES("+ai.getObject().getID()+","+oid+","+ai.getNeeded()+")");
			}
			
			// Abfrage ausfhren
			stmt.executeBatch();
		}
		catch ( BatchUpdateException e ) {
			conn.setAutoCommit(true);
			conn.rollback();
		}
		
		conn.setAutoCommit(true);
		updatecount++;
		
		// freigabe
		stmt.close();
	}
}