package de.ba_leipzig.discoinv.dialogs;

import java.awt.*;
import java.awt.event.*;
import java.sql.SQLException;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import de.ba_leipzig.discoinv.db.*;
import de.ba_leipzig.discoinv.tables.*;
import de.matthiasrothe.util.collections.*;

public abstract class AbstractAssociationDialog extends AbstractAddDialog {
	protected class DialogState {
		private boolean editing;
		
		public boolean isEditing(){
			return editing;
		}
		
		public void setEditing(boolean editing){
			this.editing = editing;
		}
	}
	
	private Insets insets = new Insets(2, 2, 2, 2);
	private JPanel jEntryPanel;
	private JPanel jDescriptionPanel;
	private JLabel jDescriptionLabel;
	private JPanel jDataEntryPanel;
	private JPanel jChooseObjectPanel;
	private JPanel jChosenObjectsPanel;
	private JComboBox jObjectComboBox;
	private JLabel jObjectLabel;
	private JLabel jNeededLabel;
	private JTextField jNeededTextField;
	private JLabel jPresentLabel;
	private JTextField jPresentTextField;
	private JButton jAddButton;
	private JScrollPane jObjectsTableScrollPane;
	private JTable jChosenObjectsTable;
	private JPanel jTableButtonPanel;
	private JButton jRemoveButton;
	private JButton jEditButton;
	private GenericTableModel chosenObjectsModel;
	protected DialogState state = new DialogState();
	
	public AbstractAssociationDialog(JFrame owner){
		super(owner, false);
	}
	
	protected void initialize(){
		super.initialize();
		fillObjectComboBox();
		getJChosenObjectsTable().requestFocus();
	}
	
	private void fillObjectComboBox(){
		try {
			doFillObjectComboBox();
			if(getJObjectComboBox().getItemCount() > 0){
				getJObjectComboBox().setSelectedIndex(0);
			}
		}
		catch(DBManagerLoadException e){
			handle(e);
		}
		catch(SQLException e){
			handle(e, "Beim Aufbau der Objektliste");
		}
	}
	
	protected JPanel getJEntryPanel(){
		if(jEntryPanel == null){
			jEntryPanel = new JPanel();
			jEntryPanel.setLayout(new BorderLayout());
			jEntryPanel.add(getJDescriptionPanel(), BorderLayout.NORTH);
			jEntryPanel.add(getJDataEntryPanel(), BorderLayout.CENTER);
		}
		return jEntryPanel;
	}
	
	protected boolean activateOkButton() {
		return chosenObjectsModel.getRowCount() > 0;
	}
	
	protected Dimension getDialogSize(){
		return new Dimension(600, 250);
	}
	
	/**
	 * This method initializes jDescriptionPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJDescriptionPanel() {
		if (jDescriptionPanel == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.insets = insets;
			
			jDescriptionPanel = new JPanel();
			jDescriptionPanel.setLayout(new GridBagLayout());
			jDescriptionPanel.setBorder(BorderFactory.createRaisedBevelBorder());
			jDescriptionPanel.add(getJDescriptionLabel(), gridBagConstraints1);
		}
		return jDescriptionPanel;
	}
	
	private JLabel getJDescriptionLabel(){
		if(jDescriptionLabel == null){
			jDescriptionLabel = new JLabel(getDescription());
		}
		return jDescriptionLabel;
	}
	
	/**
	 * This method initializes jDataEntryPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJDataEntryPanel() {
		if (jDataEntryPanel == null) {
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 0;
			gridBagConstraints2.insets = insets;
			gridBagConstraints2.fill = GridBagConstraints.HORIZONTAL;
			
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 1;
			gridBagConstraints.insets = insets;
			
			jDataEntryPanel = new JPanel();
			jDataEntryPanel.setLayout(new GridBagLayout());
			jDataEntryPanel.add(getJChooseObjectPanel(), gridBagConstraints2);
			jDataEntryPanel.add(getJChosenObjectsPanel(), gridBagConstraints);
		}
		return jDataEntryPanel;
	}
	
	/**
	 * This method initializes jChooseObjectPanel	
	 * @return JPanel 	
	 */
	private JPanel getJChooseObjectPanel() {
		if (jChooseObjectPanel == null) {
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.gridx = 3;
			gridBagConstraints9.gridy = 2;
			gridBagConstraints9.insets = insets;
			gridBagConstraints9.anchor = GridBagConstraints.WEST;
			gridBagConstraints9.weightx = 1.0;
			gridBagConstraints9.fill = GridBagConstraints.BOTH;
			
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.gridx = 2;
			gridBagConstraints8.gridy = 2;
			gridBagConstraints8.insets = insets;
			gridBagConstraints8.anchor = GridBagConstraints.WEST;
			
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.gridx = 2;
			gridBagConstraints7.gridy = 0;
			gridBagConstraints7.insets = insets;
			gridBagConstraints7.anchor = GridBagConstraints.WEST;
			
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.gridx = 1;
			gridBagConstraints6.gridy = 2;
			gridBagConstraints6.insets = insets;
			gridBagConstraints6.anchor = GridBagConstraints.WEST;
			
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.gridx = 1;
			gridBagConstraints5.gridy = 0;
			gridBagConstraints5.insets = insets;
			gridBagConstraints5.anchor = GridBagConstraints.WEST;
			
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 0;
			gridBagConstraints4.gridy = 2;
			gridBagConstraints4.insets = insets;
			gridBagConstraints4.anchor = GridBagConstraints.WEST;
			
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.gridy = 0;
			gridBagConstraints3.insets = insets;
			gridBagConstraints3.anchor = GridBagConstraints.WEST;
			
			Border border =
				BorderFactory.createTitledBorder(getChoosePanelCaption());
			
			jChooseObjectPanel = new JPanel();
			jChooseObjectPanel.setLayout(new GridBagLayout());
			jChooseObjectPanel.setBorder(border);
			jChooseObjectPanel.add(getJObjectLabel(), gridBagConstraints3);
			jChooseObjectPanel.add(getJObjectComboBox(), gridBagConstraints4);
			jChooseObjectPanel.add(getJNeededLabel(), gridBagConstraints5);
			jChooseObjectPanel.add(getJNeededTextField(), gridBagConstraints6);
			jChooseObjectPanel.add(getJPresentLabel(), gridBagConstraints7);
			jChooseObjectPanel.add(getJPresentTextField(), gridBagConstraints8);
			jChooseObjectPanel.add(getJAddButton(), gridBagConstraints9);
		}
		return jChooseObjectPanel;
	}
	
	/**
	 * This method initializes jChosenObjectsPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJChosenObjectsPanel() {
		if (jChosenObjectsPanel == null) {
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.gridx = 1;
			gridBagConstraints11.gridy = 0;
			gridBagConstraints11.insets = insets;
			gridBagConstraints11.anchor = GridBagConstraints.SOUTH;
			
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
			gridBagConstraints10.fill = GridBagConstraints.BOTH;
			gridBagConstraints10.gridy = 0;
			gridBagConstraints10.weightx = 1.0;
			gridBagConstraints10.weighty = 1.0;
			gridBagConstraints10.gridx = 0;
			gridBagConstraints10.insets = insets;
			
			Border border =
				BorderFactory.createTitledBorder(getChosenPanelCaption());
			
			jChosenObjectsPanel = new JPanel();
			jChosenObjectsPanel.setLayout(new GridBagLayout());
			jChosenObjectsPanel.setBorder(border);
			jChosenObjectsPanel.add(getJObjectsTableScrollPane(), gridBagConstraints10);
			jChosenObjectsPanel.add(getJTableButtonPanel(), gridBagConstraints11);
		}
		return jChosenObjectsPanel;
	}
	
	private JLabel getJObjectLabel(){
		if(jObjectLabel == null){
			jObjectLabel = new JLabel(getObjectCaption());
		}
		return jObjectLabel;
	}
	
	/**
	 * This method initializes jObjectComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	protected JComboBox getJObjectComboBox() {
		if (jObjectComboBox == null) {
			jObjectComboBox = new JComboBox();
			jObjectComboBox.setModel(new OrderedComboBoxModel(jObjectComboBox));
			jObjectComboBox.setPreferredSize(new Dimension(150, 20));
			jObjectComboBox.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					enableAddButton();
				}
			});
		}
		return jObjectComboBox;
	}
	
	private JLabel getJNeededLabel(){
		if(jNeededLabel == null){
			jNeededLabel = new JLabel("Bestand bentigt:");
		}
		return jNeededLabel;
	}
	
	/**
	 * This method initializes jNeededTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJNeededTextField() {
		if (jNeededTextField == null) {
			jNeededTextField = new JTextField();
			jNeededTextField.setPreferredSize(new Dimension(120, 20));
			jNeededTextField.addCaretListener(new CaretListener(){
				public void caretUpdate(CaretEvent e) {
					enableAddButton();
				}
			});
		}
		return jNeededTextField;
	}
	
	private JLabel getJPresentLabel(){
		if(jPresentLabel == null){
			jPresentLabel = new JLabel("Bestand aktuell:");
		}
		return jPresentLabel;
	}
	
	/**
	 * This method initializes jPresentTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJPresentTextField() {
		if (jPresentTextField == null) {
			jPresentTextField = new JTextField();
			jPresentTextField.setPreferredSize(new Dimension(120, 20));
			jPresentTextField.addCaretListener(new CaretListener(){
				public void caretUpdate(CaretEvent e) {
					enableAddButton();
				}
			});

		}
		return jPresentTextField;
	}
	
	/**
	 * This method initializes jAddButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJAddButton() {
		if (jAddButton == null) {
			jAddButton = new JButton(getAddButtonCaption());
			jAddButton.setEnabled(false);
			jAddButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					Nameable object =
						(Nameable)getJObjectComboBox().getSelectedItem();
					getJObjectComboBox().removeItem(object);
					
					if(getJObjectComboBox().getItemCount() > 0){
						getJObjectComboBox().setSelectedIndex(0);
					}
					
					if(activateObjectComboBox()){
						getJObjectComboBox().setEnabled(true);
					}
					
					String neededText = getJNeededTextField().getText();
					Integer needed = Integer.parseInt(neededText);
					getJNeededTextField().setText("");
					
					String presentText = getJPresentTextField().getText();
					Integer present = Integer.parseInt(presentText);
					getJPresentTextField().setText("");
					
					Vector<Object> rowData = new Vector<Object>();
					rowData.add(object);
					rowData.add(needed);
					rowData.add(present);
					
					chosenObjectsModel.addRow(rowData);
					
					state.setEditing(false);
					
					enableAllButtons();
					
					finishAddAction();
				}
			});
		}
		return jAddButton;
	}
	
	private void enableAddButton(){
		String numberRegex = "^\\d+$";
		boolean objectOK = getJObjectComboBox().getSelectedIndex() > -1;
		boolean neededOK = getJNeededTextField().getText().matches(numberRegex);
		boolean presentOK = getJPresentTextField().getText().matches(numberRegex);
		
		if(objectOK && neededOK && presentOK){
			getJAddButton().setEnabled(true);
		}
		else {
			getJAddButton().setEnabled(false);
		}
	}
	
	/**
	 * This method initializes jObjectsTableScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJObjectsTableScrollPane() {
		if (jObjectsTableScrollPane == null) {
			jObjectsTableScrollPane = new JScrollPane();
			jObjectsTableScrollPane.setViewportView(getJChosenObjectsTable());
			jObjectsTableScrollPane.setPreferredSize(new Dimension(400, 100));
		}
		return jObjectsTableScrollPane;
	}

	/**
	 * This method initializes jChosenObjectsTable	
	 * 	
	 * @return javax.swing.JTable	
	 */
	private JTable getJChosenObjectsTable() {
		if (jChosenObjectsTable == null) {
			chosenObjectsModel = new GenericTableModel(null, getColumns());
			jChosenObjectsTable = new JTable(chosenObjectsModel);
			jChosenObjectsTable.setFillsViewportHeight(true);
			jChosenObjectsTable.setAutoCreateRowSorter(true);
			jChosenObjectsTable.setSelectionMode(
					ListSelectionModel.SINGLE_SELECTION);
			
			getTableSelectionModel().addListSelectionListener(
					new ListSelectionListener(){
						public void valueChanged(ListSelectionEvent event){
							enableEditButton();
							enableRemoveButton();
						}
			});
		}
		return jChosenObjectsTable;
	}
	
	private Vector<ColumnData> getColumns(){
		Vector<ColumnData> columns = new Vector<ColumnData>();
		
		String objectCaption = getObjectCaption();
		objectCaption = objectCaption.substring(0, objectCaption.length() - 1);
		
		String neededCaption = getJNeededLabel().getText();
		neededCaption = neededCaption.substring(0, neededCaption.length() - 1);
		
		String presentCaption = getJPresentLabel().getText();
		presentCaption = presentCaption.substring(0, presentCaption.length() - 1);
		
		columns.add(new ColumnData(objectCaption, String.class));
		columns.add(new ColumnData(neededCaption, Integer.class));
		columns.add(new ColumnData(presentCaption, Integer.class));
		
		return columns;
	}
	
	/**
	 * This method initializes jTableButtonPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJTableButtonPanel() {
		if (jTableButtonPanel == null) {
			GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
			gridBagConstraints13.gridy = 1;
			gridBagConstraints13.gridx = 0;
			gridBagConstraints13.insets = insets;
			gridBagConstraints13.fill = GridBagConstraints.BOTH;
			
			GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
			gridBagConstraints12.gridx = 0;
			gridBagConstraints12.gridy = 0;
			gridBagConstraints12.insets = insets;
			
			jTableButtonPanel = new JPanel();
			jTableButtonPanel.setLayout(new GridBagLayout());
			jTableButtonPanel.add(getJEditButton(), gridBagConstraints12);
			jTableButtonPanel.add(getJRemoveButton(), gridBagConstraints13);
		}
		return jTableButtonPanel;
	}

	/**
	 * This method initializes jRemoveButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJRemoveButton() {
		if (jRemoveButton == null) {
			jRemoveButton = new JButton("Eintrag entfernen");
			jRemoveButton.setEnabled(false);
			jRemoveButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					int index = getTableSelectionModel().getMinSelectionIndex();
					
					Vector rowData =
						(Vector)getTableData().get(index);
					
					chosenObjectsModel.removeRow(index);
					
					int rows = chosenObjectsModel.getRowCount();
					
					if(addRemovedItemToComboBox()){
						getJObjectComboBox().addItem(rowData.get(0));						
					}
					
					if(rows == 0){
						if(state.isEditing()){
							
						}
						else {
							getJObjectComboBox().setSelectedItem(rowData.get(0));
							getJObjectComboBox().requestFocus();
						}
					}
					else if(rows == index){
						getTableSelectionModel()
						.setLeadSelectionIndex(index - 1);
					}
					else {
						getTableSelectionModel()
						.setLeadSelectionIndex(index);
					}
					
					enableAllButtons();
				}
			});
		}
		return jRemoveButton;
	}
	
	private void enableRemoveButton(){
		if(isRowSelected()){
			getJRemoveButton().setEnabled(true);
		}
		else {
			getJRemoveButton().setEnabled(false);
		}
	}
	
	/**
	 * This method initializes jEditButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJEditButton() {
		if (jEditButton == null) {
			jEditButton = new JButton("Eintrag bearbeiten");
			jEditButton.setEnabled(false);
			jEditButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					int index = getTableSelectionModel().getMinSelectionIndex();
					
					Vector rowData =
						(Vector)getTableData().get(index);
					
					getJObjectComboBox().addItem(rowData.get(0));
					getJObjectComboBox().setSelectedItem(rowData.get(0));
					getJObjectComboBox().setEnabled(false);
					
					getJNeededTextField().setText(rowData.get(1).toString());
					getJNeededTextField().requestFocus();
					
					getJPresentTextField().setText(rowData.get(2).toString());
					
					chosenObjectsModel.removeRow(index);
					
					enableAllButtons();
					
					state.setEditing(true);
				}
			});
		}
		return jEditButton;
	}
	
	private void enableEditButton(){
		if(isRowSelected() && !state.isEditing()){
			getJEditButton().setEnabled(true);
		}
		else {
			getJEditButton().setEnabled(false);
		}
	}
	
	private boolean isRowSelected(){
		return getTableSelectionModel().getMinSelectionIndex() > -1;
	}
	
	private Vector<Vector<Object>> getTableData(){
		return getTableModel().getDataVector();
	}
	
	protected ListSelectionModel getTableSelectionModel(){
		return getJChosenObjectsTable().getSelectionModel();
	}
	
	protected void enableAllButtons(){
		enableRemoveButton();
		enableEditButton();
		enableOkButton();
	}
	
	protected GenericTableModel getTableModel(){
		if(chosenObjectsModel == null){
			throw new IllegalStateException();
		}
		return chosenObjectsModel;
	}
	
	protected boolean activateObjectComboBox(){
		return true;
	}
	
	protected void finishAddAction(){
		if(getJObjectComboBox().getItemCount() > 0){
			getJObjectComboBox().requestFocus();
		}
		else {
			getJOkButton().requestFocus();
		}
	}
	
	protected boolean addRemovedItemToComboBox(){
		return true;
	}
	
	protected void doOkAction() {
		Vector<Vector<Object>> data = getTableData();
		Set<AssociationInfo<Identifiable>> associations =
			new ArraySet<AssociationInfo<Identifiable>>();
		
		for(Vector row : data){
			AssociationInfo<Identifiable> association =
				new AssociationInfo<Identifiable>();
			
			association.setObject((Identifiable)row.get(0));
			association.setNeeded((Integer)row.get(1));
			association.setPresent((Integer)row.get(2));
			
			associations.add(association);
		}
		
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			doDBAction(dbManager, associations);
			
			exitType = OK;
		}
		catch(DBManagerLoadException e){
			handle(e);
		}
		catch(SQLException e){
			handle(e, "Beim ndern der Zuordnungen");
		}
	}
	
	protected abstract String getDescription();
	protected abstract String getChoosePanelCaption();
	protected abstract String getChosenPanelCaption();
	protected abstract String getObjectCaption();
	protected abstract String getAddButtonCaption();
	protected abstract void doFillObjectComboBox()
		throws SQLException, DBManagerLoadException;
	protected abstract void doDBAction(DatabaseManager dbManager,
		Set<AssociationInfo<Identifiable>> associations) throws SQLException;
}