package de.ba_leipzig.discoinv.dialogs;

import java.awt.*;
import java.awt.event.*;
import java.sql.SQLException;

import javax.swing.*;

import de.ba_leipzig.discoinv.ExceptionHandler;
import de.ba_leipzig.discoinv.db.DBIntegrityException;
import de.ba_leipzig.discoinv.db.DBManagerLoadException;

public abstract class AbstractDialog extends JDialog {
	private enum ExitType {
		ok, cancel
	}
	
	public static final ExitType OK = ExitType.ok;
	public static final ExitType CANCEL = ExitType.cancel;
	
	protected ExitType exitType = CANCEL;
	private JPanel jContentPane;
	private JPanel jCtrlPanel;
	private JButton jOkButton;
	private JButton jCancelButton;
	
	/**
	 * @param owner
	 */
	public AbstractDialog(JFrame owner) {
		this(owner, true);
	}
	
	public AbstractDialog(JFrame owner, boolean init) {
		super(owner);
		if(init) initialize();		
	}
	
	public ExitType getExitType(){
		return exitType;
	}
	
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	protected void initialize() {
		this.setSize(getDialogSize());
		this.setTitle(getDialogTitle());
		this.setContentPane(getJContentPane());
		this.pack();
		
		// Positioniere Dialogfenster in der Mitte des Elternfensters
		Rectangle parentBounds = this.getParent().getBounds();
		Point location = new Point();
		location.x = (int)((parentBounds.getCenterX()) - (this.getWidth() / 2));
		location.y = (int)((parentBounds.getCenterY()) - (this.getHeight() / 2));
		this.setLocation(location);
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	protected JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJCtrlPanel(), BorderLayout.SOUTH);
			jContentPane.add(getJEntryPanel(), BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jCtrlPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	protected JPanel getJCtrlPanel() {
		if (jCtrlPanel == null) {
			jCtrlPanel = new JPanel();
			jCtrlPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
			jCtrlPanel.add(getJOkButton(), null);
			
			if(showCancelButton()){
				jCtrlPanel.add(getJCancelButton(), null);
			}
		}
		return jCtrlPanel;
	}

	/**
	 * This method initializes jOkButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	protected JButton getJOkButton() {
		if (jOkButton == null) {
			String caption = getOkButtonCaption();
			jOkButton = new JButton(caption);
			jOkButton.setMnemonic(caption.charAt(0));
			jOkButton.setEnabled(false);
			jOkButton.addActionListener(getOkButtonHandler());
		}
		return jOkButton;
	}

	/**
	 * This method initializes jCancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	protected JButton getJCancelButton() {
		if (jCancelButton == null) {
			String caption = getCancelButtonCaption();
			jCancelButton = new JButton(caption);
			jCancelButton.setMnemonic(caption.charAt(0));
			jCancelButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e){
					dispose();
				}
			});
		}
		return jCancelButton;
	}
	
	
	protected void handle(DBManagerLoadException e){
		new ExceptionHandler(this.getOwner()).handle(e);
	}
	
	protected void handle(SQLException e, String msg){
		new ExceptionHandler(this.getOwner()).handle(e, msg +
				" ist ein Datenbankfehler aufgetreten:");
	}
	
	protected void handle(DBIntegrityException e, String msg){
		new ExceptionHandler(this.getOwner()).handle(e, msg +
				" ist ein Datenbankfehler aufgetreten:");
	}
	
	protected String getCancelButtonCaption(){
		return "Abbrechen";
	}
	
	protected boolean showCancelButton(){
		return true;
	}
	
	protected abstract Dimension getDialogSize();
	protected abstract String getDialogTitle();
	protected abstract JPanel getJEntryPanel();
	protected abstract String getOkButtonCaption();
	protected abstract ActionListener getOkButtonHandler();
}