package de.ba_leipzig.discoinv.dialogs;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.SQLException;
import java.util.Set;

import javax.swing.*;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import de.ba_leipzig.discoinv.db.*;

public class AddBeverageDialog extends AbstractAddObjectDialog {
	private static final long serialVersionUID = -268954781249746094L;
	private JCheckBox jAlcCheckBox;
	private JCheckBox jRefundCheckBox;
	private JLabel jCapacityLabel;
	private JTextField jCapacityTextField;
	private JLabel jTypeLabel;
	private JComboBox jTypeComboBox;
	private JPanel jDataSubPanel;
	private JPanel jDataPanel;
	private JPanel jLevelPanel;
	private JCheckBox jLevelCheckBox;
	private JPanel jLevelSubPanel;
	private JLabel jEmptyLevelLabel;
	private JLabel jFullLevelLabel;
	private JTextField jEmptyLevelTextField;
	private JTextField jFullLevelTextField;
	
	public AddBeverageDialog(JFrame owner){
		super(owner);
		fillTypeComboBox();
	}
	
	private void fillTypeComboBox(){
		jTypeComboBox.addItem("");
		
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			Set<BeverageType> types = dbManager.getBeverageTypes();
			if(types != null) for(BeverageType type : types){
				jTypeComboBox.addItem(type);
			}
		}
		catch(DBManagerLoadException e){
			handle(e);
		}
		catch(SQLException e){
			handle(e, "Beim Aufbau der Typliste");
		}
	}
	
	protected boolean activateOkButton(){
		String numberRegex = "^[1-9]\\d*$";
		boolean capacityOK =
			getJCapacityTextField().getText().matches(numberRegex);
		boolean typeOK = getJTypeComboBox().getSelectedIndex() > 0;
		
		boolean levelOK = true;
		if(getJLevelCheckBox().isSelected()){
			String emptyLevelText = getJEmptyLevelTextField().getText();
			String fullLevelText = getJFullLevelTextField().getText();
			
			boolean emptyLevelOK = emptyLevelText.matches(numberRegex);
			boolean fullLevelOK = fullLevelText.matches(numberRegex);
			
			levelOK = emptyLevelOK && fullLevelOK;
			
			boolean fullLevelLargerThanEmptyLevel = false;
			if(levelOK){
				int emptyLevel = Integer.parseInt(emptyLevelText);
				int fullLevel = Integer.parseInt(fullLevelText);
				fullLevelLargerThanEmptyLevel = fullLevel > emptyLevel;
			}
			
			levelOK &= fullLevelLargerThanEmptyLevel;
		}
		
		return capacityOK && typeOK && levelOK;
	}
	
	protected void addMore(JPanel jEntryPanel){
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
		gridBagConstraints1.gridx = 0;
		gridBagConstraints1.gridy = 2;
		jEntryPanel.add(getJDataPanel(), gridBagConstraints);
		jEntryPanel.add(getJLevelPanel(), gridBagConstraints1);
	}
	
	protected void doOkAction(){
		boolean isBeverageWithLevel = getJLevelCheckBox().isSelected();
		
		// Beverage-Objekt vorbereiten
		Beverage beverage;
		
		if(isBeverageWithLevel){
			// Getrnk mit Fllstand
			String emptyLevelText = getJEmptyLevelTextField().getText();
			int empty = Integer.parseInt(emptyLevelText);
			String fullLevelText = getJFullLevelTextField().getText();
			int full = Integer.parseInt(fullLevelText);
			
			beverage = new BeverageWithLevel();
			fillBeverageObject(beverage);
			((BeverageWithLevel)beverage).setEmpty(empty);
			((BeverageWithLevel)beverage).setFull(full);
		}
		else {
			// Getrnk ohne Fllstand
			beverage = new Beverage();
			fillBeverageObject(beverage);
		}
		
		// Beverage-Objekt in DB einfgen
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			
			if(isBeverageWithLevel){
				doDBAction(dbManager, (BeverageWithLevel)beverage);
			}
			else {
				doDBAction(dbManager, beverage);
			}
			
			exitType = OK;
		}
		catch(DBManagerLoadException e){
			handle(e);
		}
		catch(SQLException e){
			handle(e, getDBExceptionMessage());
		}
		catch(DBIntegrityException e){
			handle(e, getDBExceptionMessage());
		}
	}
	
	protected void doDBAction(DatabaseManager dbManager, Beverage beverage)
	throws SQLException, DBIntegrityException {
		dbManager.addBeverage(beverage);
		setObject(beverage);
	}
	
	protected void doDBAction(DatabaseManager dbManager, BeverageWithLevel beverage)
	throws SQLException, DBIntegrityException {
		dbManager.addBeverage(beverage);
		setObject(beverage);
	}
	
	protected String getDBExceptionMessage(){
		return "Beim Hinzufgen eines Getrnks";
	}
	
	private void fillBeverageObject(Beverage beverage){
		String name = getJNameTextField().getText();
		boolean alc = getJAlcCheckBox().isSelected();
		boolean refund = getJRefundCheckBox().isSelected();
		String capacityText = getJCapacityTextField().getText();
		int capacity = Integer.parseInt(capacityText);
		BeverageType type = (BeverageType)getJTypeComboBox().getSelectedItem();
		beverage.setName(name);
		beverage.setAlc(alc);
		beverage.setRefund(refund);
		beverage.setCapacity(capacity);
		beverage.setType(type);		
	}
	
	protected Dimension getDialogSize(){
		return new Dimension(300, 200);
	}
	
	protected String getDialogTitle(){
		return "Getrnk hinzufgen";
	}
	
	/**
	 * This method initializes jDataPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJDataPanel() {
		if(jDataPanel == null){
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 1;
			gridBagConstraints2.gridwidth = 2;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 1;
			gridBagConstraints1.gridy = 0;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
			
			jDataPanel = new JPanel();
			jDataPanel.setLayout(new GridBagLayout());
			jDataPanel.setBorder(BorderFactory.createTitledBorder("weitere Daten"));
			jDataPanel.add(getJAlcCheckBox(), gridBagConstraints);
			jDataPanel.add(getJRefundCheckBox(), gridBagConstraints1);
			jDataPanel.add(getJDataSubPanel(), gridBagConstraints2);		
		}
		return jDataPanel;
	}
	
	/**
	 * This method initializes jAlcCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	protected JCheckBox getJAlcCheckBox() {
		if (jAlcCheckBox == null) {
			jAlcCheckBox = new JCheckBox("Alkoholisch");
			Dimension size = new Dimension(TOTAL_WIDTH / 2, NORMAL_HEIGHT);
			jAlcCheckBox.setPreferredSize(size);
		}
		return jAlcCheckBox;
	}

	/**
	 * This method initializes jRefundCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	protected JCheckBox getJRefundCheckBox() {
		if (jRefundCheckBox == null) {
			jRefundCheckBox = new JCheckBox("Pfand");
			Dimension size = new Dimension(TOTAL_WIDTH / 2, NORMAL_HEIGHT);
			jRefundCheckBox.setPreferredSize(size);

		}
		return jRefundCheckBox;
	}

	/**
	 * This method initializes jCapacityTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	protected JTextField getJCapacityTextField() {
		if (jCapacityTextField == null) {
			jCapacityTextField = new JTextField();
			jCapacityTextField.setPreferredSize(getTextFieldSize());
			
			jCapacityTextField.addCaretListener(new CaretListener(){
				public void caretUpdate(CaretEvent e){
					enableOkButton();
				}
			});
		}
		return jCapacityTextField;
	}

	/**
	 * This method initializes jTypeComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	protected JComboBox getJTypeComboBox() {
		if (jTypeComboBox == null) {
			jTypeComboBox = new JComboBox();
			jTypeComboBox.setModel(new OrderedComboBoxModel(jTypeComboBox));
			
			jTypeComboBox.setPreferredSize(getTextFieldSize());
			
			jTypeComboBox.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e){
					enableOkButton();
				}
			});
		}
		return jTypeComboBox;
	}

	/**
	 * This method initializes jDataSubPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJDataSubPanel() {
		if (jDataSubPanel == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.anchor = GridBagConstraints.WEST;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 1;
			gridBagConstraints2.gridy = 0;
			gridBagConstraints2.anchor = GridBagConstraints.EAST;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.gridy = 2;
			gridBagConstraints3.anchor = GridBagConstraints.WEST;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 1;
			gridBagConstraints4.gridy = 2;
			gridBagConstraints4.anchor = GridBagConstraints.EAST;			
			
			jDataSubPanel = new JPanel();
			int height = getLabelSize().height * 2;
			Dimension size = new Dimension(TOTAL_WIDTH, height);
			jDataSubPanel.setPreferredSize(size);
			jDataSubPanel.setLayout(new GridBagLayout());
			jDataSubPanel.add(getJCapacityLabel(), gridBagConstraints1);
			jDataSubPanel.add(getJCapacityTextField(), gridBagConstraints2);
			jDataSubPanel.add(getJTypeLabel(), gridBagConstraints3);
			jDataSubPanel.add(getJTypeComboBox(), gridBagConstraints4);
		}
		return jDataSubPanel;
	}
	
	private JLabel getJCapacityLabel(){
		if(jCapacityLabel == null){
			jCapacityLabel = new JLabel("Fllmenge (in ml):");
			jCapacityLabel.setPreferredSize(getLabelSize());
		}
		return jCapacityLabel;
	}
	
	private JLabel getJTypeLabel(){
		if(jTypeLabel == null){
			jTypeLabel = new JLabel("Typ:");
			jTypeLabel.setPreferredSize(getLabelSize());
		}
		return jTypeLabel;
	}
	
	private JPanel getJLevelPanel(){
		if(jLevelPanel == null){
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 0;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 1;
			
			jLevelPanel = new JPanel();
			jLevelPanel.setLayout(new GridBagLayout());
			jLevelPanel.setBorder(
					BorderFactory.createTitledBorder("Angaben zum Fllstand"));
			
			jLevelPanel.add(getJLevelCheckBox(), gridBagConstraints1);
			jLevelPanel.add(getJLevelSubPanel(), gridBagConstraints2);
		}
		return jLevelPanel;
	}
	
	protected JCheckBox getJLevelCheckBox(){
		if(jLevelCheckBox == null){
			jLevelCheckBox = new JCheckBox("Getrnk mit Fllstand");
			Dimension size = new Dimension(TOTAL_WIDTH, NORMAL_HEIGHT);
			jLevelCheckBox.setPreferredSize(size);
			
			jLevelCheckBox.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e){
					boolean enabled = jLevelCheckBox.isSelected();
					jEmptyLevelLabel.setEnabled(enabled);
					jEmptyLevelTextField.setEnabled(enabled);
					jFullLevelLabel.setEnabled(enabled);
					jFullLevelTextField.setEnabled(enabled);
					
					// OK Button (de)aktivieren, da die Fllstandsfelder
					// zur Entscheidungsgrundlage hinzugekommen sind
					enableOkButton();
					
					if(enabled){
						jEmptyLevelTextField.requestFocus();
					}
				}				
			});
		}
		return jLevelCheckBox;
	}
	
	private JPanel getJLevelSubPanel(){
		if(jLevelSubPanel == null){
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.gridy = 0;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 1;
			gridBagConstraints2.gridy = 0;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.gridy = 1;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 1;
			gridBagConstraints4.gridy = 1;
			
			jLevelSubPanel = new JPanel();
			jLevelSubPanel.setLayout(new GridBagLayout());
			jLevelSubPanel.add(getJEmptyLevelLabel(), gridBagConstraints1);
			jLevelSubPanel.add(getJEmptyLevelTextField(), gridBagConstraints2);
			jLevelSubPanel.add(getJFullLevelLabel(), gridBagConstraints3);
			jLevelSubPanel.add(getJFullLevelTextField(), gridBagConstraints4);
		}
		
		return jLevelSubPanel;
	}
	
	private JLabel getJEmptyLevelLabel(){
		if(jEmptyLevelLabel == null){
			jEmptyLevelLabel = new JLabel("Leer (in g):");
			jEmptyLevelLabel.setPreferredSize(getLabelSize());
			jEmptyLevelLabel.setEnabled(false);
		}
		return jEmptyLevelLabel;
	}
	
	protected JTextField getJEmptyLevelTextField(){
		if(jEmptyLevelTextField == null){
			jEmptyLevelTextField = new JTextField();
			jEmptyLevelTextField.setPreferredSize(getTextFieldSize());
			jEmptyLevelTextField.setEnabled(false);
			
			jEmptyLevelTextField.addCaretListener(new CaretListener(){
				public void caretUpdate(CaretEvent e){
					enableOkButton();
				}
			});
		}
		return jEmptyLevelTextField;
	}
	
	private JLabel getJFullLevelLabel(){
		if(jFullLevelLabel == null){
			jFullLevelLabel = new JLabel("Voll (in g):");
			jFullLevelLabel.setPreferredSize(getLabelSize());
			jFullLevelLabel.setEnabled(false);
		}
		return jFullLevelLabel;
	}
	
	protected JTextField getJFullLevelTextField(){
		if(jFullLevelTextField == null){
			jFullLevelTextField = new JTextField();
			jFullLevelTextField.setPreferredSize(getTextFieldSize());
			jFullLevelTextField.setEnabled(false);
			
			jFullLevelTextField.addCaretListener(new CaretListener(){
				public void caretUpdate(CaretEvent e){
					enableOkButton();
				}
			});			
		}
		return jFullLevelTextField;
	}
	
	private Dimension getLabelSize(){
		int width = (int)(TOTAL_WIDTH / 3);
		int height = NORMAL_HEIGHT + 2 * NORMAL_DISTANCE;
		return new Dimension(width, height);
	}
	
	private Dimension getTextFieldSize(){
		int width = TOTAL_WIDTH - getLabelSize().width;
		return new Dimension(width, NORMAL_HEIGHT);
	}
}