package de.ba_leipzig.discoinv.dialogs;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.SQLException;
import java.util.*;
import java.util.List;

import javax.swing.*;

import de.ba_leipzig.discoinv.db.*;

public class AddPlaceDialog extends AbstractAddObjectDialog {
	private static final long serialVersionUID = 1L;
	
	private JPanel jDepotPanel;
	private JCheckBox jDepotCheckBox;
	private JComboBox jDepotComboBox;
	
	/**
	 * @param owner
	 */
	public AddPlaceDialog(JFrame owner) {
		super(owner);
		fillDepotComboBox();
	}
	
	private void fillDepotComboBox() {
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			List<Place> depots = new ArrayList<Place>(dbManager.getPlaces());
			
			Collections.sort(depots, new Comparator<Place>(){
				public int compare(Place place1, Place place2){
					return place1.getName().compareTo(place2.getName());
				}
			});
			
			for(Place depot : depots){
				jDepotComboBox.addItem(depot);
			}
		}
		catch(DBManagerLoadException e){
			handle(e);
		}
		catch(SQLException e){
			handle(e, "Beim Aufbau der Depotliste");
		}
	}

	/**
	 * This method initializes jDepotPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJDepotPanel() {
		if (jDepotPanel == null) {
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 1;
			jDepotPanel = new JPanel();
			jDepotPanel.setBorder(BorderFactory.createTitledBorder("Lager"));
			jDepotPanel.setLayout(new GridBagLayout());
			jDepotPanel.add(getJDepotCheckBox(), new GridBagConstraints());
			jDepotPanel.add(getJDepotComboBox(), gridBagConstraints2);
		}
		return jDepotPanel;
	}

	/**
	 * This method initializes jDepotCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	protected JCheckBox getJDepotCheckBox() {
		if (jDepotCheckBox == null) {
			jDepotCheckBox = new JCheckBox("Lager bentigt");
			Dimension size = new Dimension(TOTAL_WIDTH, NORMAL_HEIGHT);
			jDepotCheckBox.setPreferredSize(size);
			jDepotCheckBox.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e){
					getJDepotComboBox().setEnabled(
							getJDepotCheckBox().isSelected());
				}
			});
		}
		return jDepotCheckBox;
	}

	/**
	 * This method initializes jDepotComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	protected JComboBox getJDepotComboBox() {
		if (jDepotComboBox == null) {
			jDepotComboBox = new JComboBox();
			jDepotComboBox.setModel(new OrderedComboBoxModel(jDepotComboBox));
			Dimension size = new Dimension(TOTAL_WIDTH, NORMAL_HEIGHT);
			jDepotComboBox.setPreferredSize(size);
			jDepotComboBox.setEnabled(false);
		}
		return jDepotComboBox;
	}

	protected Dimension getDialogSize() {
		return new Dimension(300, 200);
	}

	protected String getDialogTitle() {
		return "Standort hinzufgen";
	}

	protected void addMore(JPanel jEntryPanel) {
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 1;
		jEntryPanel.add(getJDepotPanel(), gridBagConstraints);
	}

	protected void doOkAction() {
		Place place = new Place();
		place.setName(getJNameTextField().getText());
		
		if(jDepotCheckBox.isSelected()){
			Place depot = (Place)jDepotComboBox.getSelectedItem();
			if(depot != null){
				place.setDepotID(depot.getID());
			}
		}
		
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			doDBAction(dbManager, place);
			exitType = OK;
		}
		catch(SQLException e){
			handle(e, getDBExceptionMessage());
		}
		catch(DBManagerLoadException e){
			handle(e);
		}
		catch(DBIntegrityException e){
			handle(e, getDBExceptionMessage());
		}
	}
	
	protected void doDBAction(DatabaseManager dbManager, Place place)
	throws SQLException, DBIntegrityException {
		dbManager.addPlace(place);
		setObject(place);
	}
	
	protected String getDBExceptionMessage(){
		return "Beim Hinzufgen des Standorts";
	}
}