package de.ba_leipzig.discoinv.dialogs;

import java.sql.SQLException;
import java.util.*;

import javax.swing.*;

import de.ba_leipzig.discoinv.db.*;
import de.ba_leipzig.discoinv.tables.*;

public class EditBeverageAssociationDialog extends AbstractEditAssociationDialog {
	private static final long serialVersionUID = 621285821833751639L;
	private DialogCaptions captions = new DialogCaptions();
	private Place place;
	private Set<? extends AssociationInfo<? extends InvObject>> objectInfos;
	
	public EditBeverageAssociationDialog(JFrame owner, Place place) {
		super(owner);
		
		if(place == null){
			throw new IllegalArgumentException("place must not be null!");
		}
		
		this.place = place;
		initialize();
	}
	
	protected String getChoosePanelCaption() {
		return captions.getBeverageChoosePanelCaption();
	}

	protected String getChosenPanelCaption() {
		return captions.getBeverageChosenPanelCaption();
	}

	protected String getDescription() {
		return "Getrnke in " + place.getName() + " bearbeiten";
	}

	protected String getObjectCaption() {
		return captions.getBeverageObjectCaption();
	}

	protected String getOkButtonCaption() {
		return captions.getEditOkButtonCaption();
	}
	
	protected String getDialogTitle() {
		return "Zuordnungen bearbeiten: Getrnke";
	}

	protected String getAddButtonCaption() {
		return captions.getAddButtonCaptionForEdit();
	}

	protected void fillObjectsTable(){
		try {
			GenericTableModel model = getTableModel();
			Set<? extends AssociationInfo<? extends InvObject>> objectInfoSet =
				getObjectInfos();
			
			if(objectInfoSet != null && objectInfoSet.size() > 0){
				for(AssociationInfo<? extends InvObject> objectInfo : objectInfoSet){
					InvObject object = objectInfo.getObject();
					if(object instanceof Beverage){
						Vector<Object> rowData = new Vector<Object>();
						
						rowData.add(object);
						rowData.add(objectInfo.getNeeded());
						rowData.add(objectInfo.getPresent());
						
						int position = getEntryPosition(object.getName());
						model.insertRow(position, rowData);						
					}
				}
			}
		}
		catch(DBManagerLoadException e){
			handle(e);
		}
		catch(SQLException e){
			handle(e, "Beim Aufbau der Objekttabelle");
		}
	}
	
	private Set<? extends AssociationInfo<? extends InvObject>> getObjectInfos()
	throws DBManagerLoadException, SQLException{
		if(objectInfos == null){
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			PlaceWithObjects place =
				dbManager.getPlaceWithObjects(this.place.getID());
			objectInfos = place.getObjectInfo();
		}
		return objectInfos;
	}

	protected void doDBAction(DatabaseManager dbManager,
			Set<AssociationInfo<Identifiable>> associations)
	throws SQLException {
		dbManager.updateObjectAssociations(place, associations);
	}
}