package de.ba_leipzig.discoinv.dialogs;

import java.sql.SQLException;
import java.util.*;

import javax.swing.*;

import de.ba_leipzig.discoinv.db.*;
import de.ba_leipzig.discoinv.tables.*;

public class EditPlaceAssociationDialog extends AbstractEditAssociationDialog {
	private static final long serialVersionUID = -7094575683051037708L;
	private DialogCaptions captions = new DialogCaptions();
	private InvObject object;
	
	public EditPlaceAssociationDialog(JFrame owner, InvObject object) {
		super(owner);
		
		if(object == null){
			throw new IllegalArgumentException("object must not be null!");
		}
		
		this.object = object;
		initialize();
	}
	
	protected String getChoosePanelCaption() {
		return captions.getPlaceChoosePanelCaption();
	}
	
	protected String getChosenPanelCaption() {
		return captions.getPlaceChosenPanelCaption();
	}
	
	protected String getDescription() {
		return object.getName() + " in Standorten bearbeiten";
	}
	
	protected String getObjectCaption() {
		return captions.getPlaceObjectCaption();
	}
	
	protected String getOkButtonCaption() {
		return captions.getEditOkButtonCaption();
	}
	
	protected String getDialogTitle() {
		return "Zuordnungen bearbeiten: Standorte";
	}

	protected String getAddButtonCaption() {
		return captions.getAddButtonCaptionForEdit();
	}

	protected void fillObjectsTable() {
		try {
			GenericTableModel model = getTableModel();
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			Set<? extends AssociationInfo<? extends Place>> associatedPlaces =
				dbManager.getAssociatedPlaces(object);
			
			if(associatedPlaces != null && associatedPlaces.size() > 0){
				for(AssociationInfo<? extends Place> associatedPlace :
					associatedPlaces){
					
					Place place = associatedPlace.getObject();
					Vector<Object> rowData = new Vector<Object>();
					
					rowData.add(place);
					rowData.add(associatedPlace.getNeeded());
					rowData.add(associatedPlace.getPresent());
					
					int position = getEntryPosition(place.getName());
					model.insertRow(position, rowData);						
				}
			}
		}
		catch(DBManagerLoadException e){
			handle(e);
		}
		catch(SQLException e){
			handle(e, "Beim Aufbau der Standorttabelle");
		}
	}

	protected void doDBAction(DatabaseManager dbManager,
			Set<AssociationInfo<Identifiable>> associations)
	throws SQLException {
		dbManager.updatePlaceAssociations(object, associations);
	}
}