package de.ba_leipzig.discoinv.tables;

import java.sql.SQLException;
import java.util.*;

import javax.swing.*;
import javax.swing.table.TableRowSorter;

import de.ba_leipzig.discoinv.db.DatabaseManager;

public abstract class AbstractTable implements Table {
	private GenericTableModel model;
	private int dbManagerID = 0;
	private int dbUpdateCount = 0;
	
	public GenericTableModel getTableModel(DatabaseManager dbManager)
	throws SQLException {
		if(dbManager == null){
			throw new IllegalArgumentException("dbManager must not be null!");
		}
		
		if(model != null && dbNotChanged(dbManager)){
			return model;
		}
		
		dbManagerID = dbManager.getDBManagerID();
		dbUpdateCount = dbManager.getDBUpdateCount();
		
		model = doGetTableModel(dbManager);
		
		return model;
	}
	
	public RowSorter<GenericTableModel> getRowSorter(){
		RowSorter<GenericTableModel> sorter =
			new TableRowSorter<GenericTableModel>(model);
		sorter.setSortKeys(getSortKeys());
		return sorter;
	}
	
	protected List<RowSorter.SortKey> getSortKeys(){
		List<RowSorter.SortKey> sortKeys = new ArrayList<RowSorter.SortKey>();
		sortKeys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
		return sortKeys;
	}
	
	protected abstract GenericTableModel doGetTableModel(
			DatabaseManager dbManager) throws SQLException;
	
	private boolean dbNotChanged(DatabaseManager dbManager){
		return (dbManager.getDBManagerID() == dbManagerID &&
				dbManager.getDBUpdateCount() == dbUpdateCount);
	}
}