package de.ba_leipzig.discoinv.tables;

import java.sql.SQLException;
import java.util.*;

import javax.swing.RowSorter;
import javax.swing.SortOrder;

import de.ba_leipzig.discoinv.db.*;
import de.matthiasrothe.util.measuring.*;
import de.matthiasrothe.util.rating.*;

public class AllBeveragesTable extends AbstractTable implements Table {
	private Vector<ColumnData> columnData = new Vector<ColumnData>();
	private RatingColumnData ratingColumn = 
		new RatingColumnData("Gesamtberschuss", Rateable.class,
				new RateableRatingProcessor());
	
	public AllBeveragesTable(){
		initColumnData();
	}
	
	protected List<RowSorter.SortKey> getSortKeys(){
		List<RowSorter.SortKey> sortKeys = new ArrayList<RowSorter.SortKey>();
		sortKeys.add(new RowSorter.SortKey(4, SortOrder.DESCENDING));
		sortKeys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
		return sortKeys;
	}
	
	protected GenericTableModel doGetTableModel(DatabaseManager dbManager)
	throws SQLException {
		return 
		new RatingTableModel(getData(dbManager), columnData, ratingColumn);
	}
	
	private Vector<Vector<Object>> getData(DatabaseManager dbManager)
	throws SQLException {
		Set<Beverage> beverages = dbManager.getBeverages();
		Vector<Vector<Object>> data = new Vector<Vector<Object>>();
		
		if(beverages != null) for(Beverage beverage : beverages){
			int present = dbManager.getTotalPresent(beverage);
			int needed = dbManager.getTotalNeeded(beverage);
			
			Vector<Object> element = new Vector<Object>();
			element.add(beverage.getName());
			element.add(beverage.getType().getName());
			element.add(present);
			element.add(needed);
			
			element.add(new Rateable<Integer>(Validator.getInvObjectRating(
						dbManager, beverage), present - needed,	true,
						Rateable.SortOrder.DESC));
			
			element.add(beverage.isAlc());
			element.add(beverage.isRefund());
			
			double capacity = beverage.getCapacity();
			Liter unit; 
			
			if(capacity < 1000){
				unit = new Liter(MeasuringUnit.Prefix.milli);
			}
			else {
				unit = new Liter();
				capacity /= 1000d;
			}
			
			element.add(new Liters(capacity, unit));
			
			if(beverage instanceof BeverageWithLevel){
				double full = ((BeverageWithLevel)beverage).getFull();
				double empty = ((BeverageWithLevel)beverage).getEmpty();
				
				element.add(new Grams(full,	new Gram()));				
				element.add(new Grams(empty, new Gram()));
			}
			else {
				element.add(null);
				element.add(null);
			}
			
			data.add(element);
		}
		
		return data;
	}
	
	private void initColumnData(){
		columnData.add(new ColumnData("Bezeichnung", String.class));
		columnData.add(new ColumnData("Typ", String.class));
		columnData.add(new ColumnData("Gesamtbestand", Integer.class));
		columnData.add(new ColumnData("Gesamtbedarf", Integer.class));
		columnData.add(ratingColumn);
		columnData.add(new ColumnData("Alkoholisch", Boolean.class));
		columnData.add(new ColumnData("Pfand", Boolean.class));
		columnData.add(new ColumnData("Fllmenge", Measurable.class));
		columnData.add(new ColumnData("Gewicht (voll)", Measurable.class));
		columnData.add(new ColumnData("Gewicht (leer)", Measurable.class));
	}
}