package de.ba_leipzig.discoinv.tables;

import java.sql.SQLException;
import java.util.*;

import javax.swing.RowSorter;
import javax.swing.SortOrder;

import de.ba_leipzig.discoinv.db.*;
import de.matthiasrothe.util.rating.*;

public class AllMiscTable extends AbstractTable implements Table {
	private Vector<ColumnData> columnData = new Vector<ColumnData>();
	private RatingColumnData ratingColumn =
		new RatingColumnData("Gesamtberschuss", Rateable.class,
				new RateableRatingProcessor());
	
	public AllMiscTable(){
		initColumnData();
	}
	
	protected List<RowSorter.SortKey> getSortKeys(){
		List<RowSorter.SortKey> sortKeys = new ArrayList<RowSorter.SortKey>();
		sortKeys.add(new RowSorter.SortKey(3, SortOrder.DESCENDING));
		sortKeys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
		return sortKeys;
	}
	
	protected GenericTableModel doGetTableModel(DatabaseManager dbManager)
	throws SQLException {
		return
		new RatingTableModel(getData(dbManager), columnData, ratingColumn);
	}
	
	private Vector<Vector<Object>> getData(DatabaseManager dbManager)
	throws SQLException {
		Vector<Vector<Object>> data = new Vector<Vector<Object>>();
		Set<InvObject> miscSet = dbManager.getMisc();
		
		if(miscSet != null) for(InvObject misc : miscSet){
			int present = dbManager.getTotalPresent(misc);
			int needed = dbManager.getTotalNeeded(misc);
			
			Vector<Object> element = new Vector<Object>();
			element.add(misc.getName());
			element.add(present);
			element.add(needed);
			
			element.add(new Rateable<Integer>(Validator.getInvObjectRating(
						dbManager, misc), present - needed, true,
						Rateable.SortOrder.DESC));
			
			data.add(element);
		}
		
		return data;
	}
	
	private void initColumnData(){
		columnData.add(new ColumnData("Bezeichnung", String.class));
		columnData.add(new ColumnData("Gesamtbestand", Integer.class));
		columnData.add(new ColumnData("Gesamtbedarf", Integer.class));
		columnData.add(ratingColumn);
	}
}