package de.ba_leipzig.discoinv.tables;

import java.sql.SQLException;
import java.util.*;

import javax.swing.RowSorter;
import javax.swing.SortOrder;

import de.ba_leipzig.discoinv.db.*;
import de.matthiasrothe.util.rating.Rateable;
import de.matthiasrothe.util.rating.RateableRatingProcessor;
import de.matthiasrothe.util.rating.Rating;

public class AllPlacesTable extends AbstractTable implements Table {
	private Vector<ColumnData> columnData = new Vector<ColumnData>();
	private RatingColumnData ratingColumn =
		new RatingColumnData("Status", Rateable.class,
				new RateableRatingProcessor());
	
	public AllPlacesTable(){
		initColumnData();
	}
	
	protected List<RowSorter.SortKey> getSortKeys(){
		List<RowSorter.SortKey> sortKeys = new ArrayList<RowSorter.SortKey>();
		sortKeys.add(new RowSorter.SortKey(2, SortOrder.DESCENDING));
		sortKeys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
		return sortKeys;
	}
	
	protected GenericTableModel doGetTableModel(DatabaseManager dbManager)
	throws SQLException {
		return
		new RatingTableModel(getData(dbManager), columnData, ratingColumn);
	}
	
	private Vector<Vector<Object>> getData(DatabaseManager dbManager)
	throws SQLException {
		Vector<Vector<Object>> data = new Vector<Vector<Object>>();
		Set<PlaceWithObjects> places = dbManager.getPlacesWithObjects();
		
		if(places != null) for(PlaceWithObjects place : places){
			Vector<Object> element = new Vector<Object>();
			
			element.add(place.getName());
			element.add(getDepotName(place, places));
			element.add(getStatus(place));
			
			data.add(element);
		}
		
		return data;
	}
	
	private Rateable<String> getStatus(PlaceWithObjects place){
		Rating rating = Validator.getPlaceRating(place);
		String status = "OK";
		
		switch(rating){
			case warning:
				status = Validator.placeWarningStatus;
				break;
			case critic:
				status = Validator.criticStatus;
				break;
			case fatal:
				status = Validator.fatalStatus;
				break;
		}
		
		return new Rateable<String>(rating, status);
	}

	private String getDepotName(Place place, Set<? extends Place> places){
		if(place.isDepot()) return "-";
		
		String depotName = null;
		int depotID = place.getDepotID();
		
		for(Place depot : places){
			if(depot.getID() == depotID){
				depotName = depot.getName();
				break;
			}
		}
		
		return depotName;
	}

	private void initColumnData(){
		columnData.add(new ColumnData("Bezeichnung", String.class));
		columnData.add(new ColumnData("Lager", String.class));
		columnData.add(ratingColumn);
	}
}