package de.ba_leipzig.discoinv.tables;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.table.*;

import de.matthiasrothe.util.measuring.*;
import de.matthiasrothe.util.rating.*;

public class CustomJTable extends JTable {
	private class RateableRenderer implements TableCellRenderer {
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column)
		{
			Rateable rateable = (Rateable)value;
			Class<?> type = rateable.getPayload().getClass();
			Object payload = rateable.getPayload();
			
			TableCellRenderer renderer = table.getDefaultRenderer(type);
			
			Component component = renderer.getTableCellRendererComponent(table,
									payload, isSelected, hasFocus, row, column);
			
			return component;
		}
	}
	
	private class MeasurableRenderer
	extends DefaultTableCellRenderer.UIResource {
		private static final long serialVersionUID = -3178136538932616145L;

		public MeasurableRenderer(){
		    super();
		    setHorizontalAlignment(JLabel.RIGHT);
		}
		
		public void setValue(Object value){
			setText((value == null) ? "-" :
				((Measurable)value).getValueAndUnit(
						Measurable.RetrievalMode.abbreviated));
		}
	}
	
	private static final long serialVersionUID = -6441072552539144156L;
	
	public CustomJTable(GenericTableModel model){
		super(model);
	}
	
	protected void createDefaultRenderers(){
		super.createDefaultRenderers();
		setDefaultRenderer(Measurable.class, new MeasurableRenderer());
		setDefaultRenderer(Rateable.class, new RateableRenderer());
	}
	
	@SuppressWarnings("serial")
	protected JTableHeader createDefaultTableHeader(){
		return new JTableHeader(columnModel){
			public String getToolTipText(MouseEvent event){
				GenericTableModel model = (GenericTableModel)dataModel;
				Point point = event.getPoint();
				int index = columnModel.getColumnIndexAtX(point.x);
				int realIndex = 
					columnModel.getColumn(index).getModelIndex();
				return model.getColumnToolTip(realIndex);
			}
		};
	}
	
	public TableCellRenderer getCellRenderer(int row, int column){
		GenericTableModel model = (GenericTableModel)dataModel;
		
		if(model instanceof RatingTableModel){
			RatingTableModel ratingModel = (RatingTableModel)model;
			int actualRow = convertRowIndexToModel(row);
			int actualColumn = ratingModel.getRatingColumn();
			Object ratingValue = model.getValueAt(actualRow, actualColumn);
			RatingProcessor processor = ratingModel.getRatingProcessor();
			
			try {
				Rating rating = processor.getRating(ratingValue);
				return new RatingCellRenderer(rating);
			}
			catch(ProcessingException e){
				return new RatingCellRenderer(e.getClass());
			}
		}
		else {
			return super.getCellRenderer(row, column);
		}
	}
	
	public void setModel(GenericTableModel model){
		super.setModel(model);
	}
	
	public void setModel(TableModel model){
		try {
			setModel((GenericTableModel)model);
		}
		catch(ClassCastException e){
			throw new IllegalArgumentException("model must be of type GenericTableModel!");
		}
	}
}