package de.ba_leipzig.discoinv.tables;

import java.awt.*;

import javax.swing.*;
import javax.swing.table.*;

import de.matthiasrothe.util.rating.ProcessingException;
import de.matthiasrothe.util.rating.Rating;

public class RatingCellRenderer implements TableCellRenderer {
	private static final long serialVersionUID = -7650705993223579498L;
	private Rating rating;
	private boolean isException;
	
	public RatingCellRenderer(Rating rating){
		if(rating == null){
			throw new IllegalArgumentException("rating must not be null!");
		}
		
		this.rating = rating;
	}
	
	public RatingCellRenderer(Class<? extends ProcessingException> exception){
		isException = true;
	}
	
	public Component getTableCellRendererComponent(
            JTable table, Object value, boolean isSelected, boolean hasFocus,
            int row, int column){
		
		Class<?> columnClass = table.getModel().getColumnClass(column);
		TableCellRenderer renderer = table.getDefaultRenderer(columnClass);
		
		Component component = renderer.getTableCellRendererComponent(table,
								value, isSelected, hasFocus, row, column);
		
		if(!isSelected){
			if(component instanceof JComponent){
				((JComponent)component).setOpaque(true);
			}
			
			if(!isException){
				switch(rating){
					case normal:
						component.setBackground(Color.GREEN);
						break;
					case warning:
						component.setBackground(Color.ORANGE);
						break;
					case critic:
						component.setBackground(Color.YELLOW);
						break;
					case fatal:
						component.setBackground(Color.RED);
						break;	
				}
			}
			else {
				component.setBackground(Color.GRAY);
			}
		}
		
		return component;
	}
}