package de.ba_leipzig.discoinv.tables;

import java.sql.SQLException;
import java.util.*;

import javax.swing.RowSorter;
import javax.swing.SortOrder;

import de.ba_leipzig.discoinv.db.*;
import de.matthiasrothe.util.rating.Rateable;
import de.matthiasrothe.util.rating.RateableRatingProcessor;
import de.matthiasrothe.util.rating.Rating;

public class RootTable extends AbstractTable implements Table {
	private Vector<ColumnData> columnData = new Vector<ColumnData>();
	private RatingColumnData ratingColumn =
		new RatingColumnData("Bemerkung", Rateable.class,
				new RateableRatingProcessor());
	
	public RootTable(){
		initColumnData();
	}
	
	protected List<RowSorter.SortKey> getSortKeys(){
		List<RowSorter.SortKey> sortKeys = new ArrayList<RowSorter.SortKey>();
		sortKeys.add(new RowSorter.SortKey(2, SortOrder.DESCENDING));
		sortKeys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
		return sortKeys;
	}
	
	protected GenericTableModel doGetTableModel(DatabaseManager dbManager)
	throws SQLException {
		return
		new RatingTableModel(getData(dbManager), columnData, ratingColumn);
	}
	
	private Vector<Vector<Object>> getData(DatabaseManager dbManager)
	throws SQLException {
		Vector<Vector<Object>> data = new Vector<Vector<Object>>();
		data.add(getPlacesInfo(dbManager));
		data.add(getBeveragesInfo(dbManager));
		data.add(getMiscInfo(dbManager));
		
		return data;
	}
	
	private Vector<Object> getBeveragesInfo(DatabaseManager dbManager)
	throws SQLException	{
		Set<Beverage> beverages = dbManager.getBeverages();
		
		Vector<Object> beveragesInfo = new Vector<Object>();
		beveragesInfo.add("Getrnke");
		beveragesInfo.add(beverages.size());
		beveragesInfo.add(getInvObjectStatus(dbManager, beverages, false));
		
		return beveragesInfo;
	}
	
	private Vector<Object> getMiscInfo(DatabaseManager dbManager)
	throws SQLException {
		Set<InvObject> misc = dbManager.getMisc();
		
		Vector<Object> miscInfo = new Vector<Object>();
		miscInfo.add("Sonstiges");
		miscInfo.add(misc.size());
		miscInfo.add(getInvObjectStatus(dbManager, misc, true));
		
		return miscInfo;		
	}
	
	private Rateable<String> getInvObjectStatus(DatabaseManager dbManager,
								Set<? extends InvObject> objects, boolean miscCalling)
	throws SQLException {
		
		int warning = 0;
		int critic = 0;
		int fatal = 0;
		boolean validated = false;
		
		for(InvObject object : objects){
			validated = true;
			switch(Validator.getInvObjectRating(dbManager, object)){
			case warning:
				warning++;
				break;
			case critic:
				critic++;
				break;
			case fatal:
				fatal++;
				break;
			}
		}
		
		String status = "";		
		Rating rating;
		
		if(validated){
			rating = Rating.normal;
			
			if(fatal > 0){
				rating = Rating.fatal;
				status += fatal + "x " + Validator.fatalStatus;
			}
			if(fatal > 0 && critic > 0){
				status += ", ";
			}
			if(critic > 0){
				rating = Rating.critic;
				status += critic + "x " + Validator.criticStatus;
			}
			if((fatal > 0 && warning > 0) || (critic > 0 && warning > 0)){
				status += ", ";
			}
			if(warning > 0){
				rating = Rating.warning;
				status += warning + "x " + Validator.invObjectWarningStatus;
			}
		}
		else {
			rating = Rating.warning;
			status =
			"Keine " + (miscCalling ? "Sonstige" : "Getrnke") + " vorhanden";
		}
		
		return new Rateable<String>(rating, status);
	} 
	
	private Vector<Object> getPlacesInfo(DatabaseManager dbManager)
	throws SQLException {
		Set<PlaceWithObjects> places = dbManager.getPlacesWithObjects();
		
		Vector<Object> placesInfo = new Vector<Object>();
		placesInfo.add("Standorte");
		placesInfo.add(places.size());
		placesInfo.add(getPlacesStatus(places));
		
		return placesInfo;
	}
	
	private Rateable<String> getPlacesStatus(Set<PlaceWithObjects> places){
		int warning = 0;
		int critic = 0;
		int fatal = 0;
		boolean validated = false;
		
		for(PlaceWithObjects place : places){
			validated = true;
			switch(Validator.getPlaceRating(place)){
				case warning:
					warning++;
					break;
				case critic:
					critic++;
					break;
				case fatal:
					fatal++;
					break;
			}
		}
		
		String status = "";		
		Rating rating;
		
		if(validated){
			rating = Rating.normal;
			
			if(fatal > 0){
				rating = Rating.fatal;
				status += fatal + "x " + Validator.fatalStatus;
			}
			if(fatal > 0 && critic > 0){
				status += ", ";
			}
			if(critic > 0){
				rating = Rating.critic;
				status += critic + "x " + Validator.criticStatus;
			}
			if((fatal > 0 && warning > 0) || (critic > 0 && warning > 0)){
				status += ", ";
			}
			if(warning > 0){
				rating = Rating.warning;
				status += warning + "x " + Validator.placeWarningStatus;
			}
		}
		else {
			rating = Rating.warning;
			status = "Keine Standorte vorhanden";
		}
		
		return new Rateable<String>(rating, status);
	}
	
	private void initColumnData(){
		columnData.add(new ColumnData("Objekttyp", String.class));
		columnData.add(new ColumnData("Anzahl", Integer.class));
		columnData.add(ratingColumn);
	}
}