package de.ba_leipzig.discoinv.tables;

import java.sql.SQLException;
import java.util.*;

import javax.swing.RowSorter;
import javax.swing.SortOrder;

import de.ba_leipzig.discoinv.db.*;
import de.matthiasrothe.util.rating.DifferenceRatingProcessor;

public class SinglePlaceTable extends SingleObjectTable implements Table {
	private Vector<ColumnData> columnData = new Vector<ColumnData>();
	private RatingColumnData ratingColumn =
		new RatingColumnData("berschuss", Integer.class,
				new DifferenceRatingProcessor(0));
	
	public SinglePlaceTable(int id){
		super(id);
		initColumnNames();
	}
	
	protected List<RowSorter.SortKey> getSortKeys(){
		List<RowSorter.SortKey> sortKeys = new ArrayList<RowSorter.SortKey>();
		sortKeys.add(new RowSorter.SortKey(4, SortOrder.ASCENDING));
		sortKeys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
		sortKeys.add(new RowSorter.SortKey(1, SortOrder.ASCENDING));
		return sortKeys;
	}
	
	protected GenericTableModel doGetTableModel(DatabaseManager dbManager)
	throws SQLException {
		return
		new RatingTableModel(getData(dbManager), columnData, ratingColumn);
	}
	
	private Vector<Vector<Object>> getData(DatabaseManager dbManager)
	throws SQLException {
		Vector<Vector<Object>> data = new Vector<Vector<Object>>();
		PlaceWithObjects place = dbManager.getPlaceWithObjects(id);
		Set<? extends AssociationInfo<? extends InvObject>> objectInfoSet =
			place.getObjectInfo();
		
		if(objectInfoSet != null)
			for(AssociationInfo<? extends InvObject> objectInfo : objectInfoSet){
				Vector<Object> element = new Vector<Object>();
				InvObject object = objectInfo.getObject();

				if(object instanceof Beverage){
					element.add("Getrnk");
				}
				else {
					element.add("Sonstiges");
				}

				int present = objectInfo.getPresent();
				int needed = objectInfo.getNeeded();

				element.add(object.getName());
				element.add(present);
				element.add(needed);
				element.add(present - needed);

				data.add(element);
			}
		
		return data;
	}
	
	private void initColumnNames(){
		columnData.add(new ColumnData("Objekttyp", String.class));
		columnData.add(new ColumnData("Bezeichnung", String.class));
		columnData.add(new ColumnData("Bestand", Integer.class));
		columnData.add(new ColumnData("Bedarf", Integer.class));
		columnData.add(ratingColumn);
	}
}