package de.ba_leipzig.discoinv.tables;

import java.sql.SQLException;

import javax.swing.RowSorter;
import javax.swing.tree.TreePath;

import de.ba_leipzig.discoinv.db.*;
import de.ba_leipzig.discoinv.tree.*;

public class TableManager {
	private CustomJTable contentTable = null;
	private DatabaseManager dbManager = null;
	private Table rootTable = new RootTable();
	private Table placeTable = new AllPlacesTable();
	private Table beverageTable = new AllBeveragesTable();
	private Table miscTable = new AllMiscTable();
	
	public void setContentTable(CustomJTable contentTable){
		if(contentTable == null){
			throw new IllegalArgumentException("contentTable must not be null!");
		}
		
		this.contentTable = contentTable;
	}
	
	public void setDatabaseManager(DatabaseManager dbManager){
		if(dbManager == null){
			throw new IllegalArgumentException("dbManager must not be mull!");
		}
		
		this.dbManager = dbManager;
	}
	
	public void showTableForPath(TreePath path) throws SQLException {
		if(contentTable == null){
			throw new IllegalStateException("contentTable is not set!");
		}
		if(path == null){
			throw new IllegalArgumentException("path must not be null!");
		}
		
		Object[] elements = path.getPath();
		
		if(elements == null || elements.length == 0){
			throw new IllegalArgumentException("path must not be empty!");
		}
		
		switch(elements.length){
			case 1:
				showRootTable();
				break;
			case 2:
				showBaseTable(elements);
				break;
			case 3:
				showContentTable(elements);
				break;
			default:
				throw new IllegalArgumentException(
						"The length of the path is out of its bounds.");
		}
	}
	
	private TreeObjectType getTreeObjectType(TreeNode node){
		return node.getUserObject().getType();
	}
	
	private void showBaseTable(Object[] elements)
	throws SQLException {
		TreeObjectType type = getTreeObjectType((TreeNode)elements[1]);
		GenericTableModel model = null;
		RowSorter<GenericTableModel> sorter = null;
		
		switch(type){
			case place:
				model = placeTable.getTableModel(dbManager);
				sorter = placeTable.getRowSorter();
				break;
			case beverage:
				model = beverageTable.getTableModel(dbManager);
				sorter = beverageTable.getRowSorter();
				break;
			case misc:
				model = miscTable.getTableModel(dbManager);
				sorter = miscTable.getRowSorter();
				break;
			default:
				throw new RuntimeException();
		}
		
		contentTable.setModel(model);
		contentTable.setRowSorter(sorter);
	}
	
	private void showContentTable(Object[] elements) throws SQLException {
		TreeObjectType type = getTreeObjectType((TreeNode)elements[1]);
		int id = ((Identifiable)((TreeNode)elements[2]).getUserObject().
				getObject()).getID();
		
		Table table = TableFactory.getTable(type, id);
		GenericTableModel model = table.getTableModel(dbManager);
		RowSorter<GenericTableModel> sorter = table.getRowSorter();
		
		contentTable.setModel(model);
		contentTable.setRowSorter(sorter);
	}
	
	private void showRootTable() throws SQLException {
		contentTable.setModel(rootTable.getTableModel(dbManager));
		contentTable.setRowSorter(rootTable.getRowSorter());
	}
}