package de.ba_leipzig.discoinv.tables;

import java.sql.SQLException;
import java.util.Set;

import de.ba_leipzig.discoinv.db.*;
import de.matthiasrothe.util.rating.DifferenceRatingProcessor;
import de.matthiasrothe.util.rating.Rating;

public class Validator {
	public static final String placeWarningStatus = "keine Objekte vorhanden";
	public static final String invObjectWarningStatus = "keinem Standort zugeordnet";
	public static final String criticStatus = "Bestand knapp";
	public static final String fatalStatus = "Bestand fehlt";
	
	private Validator(){}
	
	public static Rating getInvObjectRating(
							DatabaseManager dbManager, InvObject object)
	throws SQLException {
		if(dbManager.hasPlaceAssociations(object)){
			
			int needed = dbManager.getTotalNeeded(object);
			int present = dbManager.getTotalPresent(object);
			int difference = present - needed;
			
			return new DifferenceRatingProcessor(0).getRating(difference);
		}
		else {
			return Rating.warning;
		}
	}
	
	public static Rating getPlaceRating(PlaceWithObjects place){
		Rating rating = Rating.normal;
		Set<? extends AssociationInfo<? extends InvObject>> objectInfoSet =
			place.getObjectInfo();
		
		if(objectInfoSet == null || objectInfoSet.size() == 0){
			return Rating.warning;
		}
		
		for(AssociationInfo objectInfo : objectInfoSet){
			int needed = objectInfo.getNeeded();
			int present = objectInfo.getPresent();
			
			if(present < needed){
				rating = Rating.fatal;
				break;
			}
			else if(present == needed){
				rating = Rating.critic;
			}
		}
		
		return rating;
	}
}