package de.ba_leipzig.discoinv.tree;

import java.awt.*;
import java.sql.SQLException;

import javax.swing.tree.TreePath;

import de.ba_leipzig.discoinv.ExceptionHandler;
import de.ba_leipzig.discoinv.db.*;

public class RenameNodeHandler implements TreePathValueChangeListener {
	private Component ui;
	
	public RenameNodeHandler(Component ui){
		if(ui == null){
			throw new IllegalArgumentException("ui must not be null!");
		}
		
		this.ui = ui;
	}
	
	public void valueForPathChanged(TreePath path, String newValue){
		Cursor cursor = ui.getCursor();
		ui.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		
		try {
			TreeElement element =
				((TreeNode)path.getLastPathComponent()).getUserObject();
			
			updateDB(element, newValue);
			updateNode(element, newValue);
		}
		catch(SQLException e){
			String msg = "Beim Umbenennen ist ein Datenbankfehler aufgetreten:";
			new ExceptionHandler(ui).handle(e, msg);
		}
		catch(DBManagerLoadException e){
			new ExceptionHandler(ui).handle(e);
		}
		finally {
			ui.setCursor(cursor);			
		}
	}
	
	private void updateDB(TreeElement element, String newValue)
	throws DBManagerLoadException, SQLException {
		int id = ((Identifiable)element.getObject()).getID();
		DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();

		switch(element.getType()){
		case place:
			dbManager.renamePlace(id, newValue);
			break;
		case beverage:
		case misc:
			dbManager.renameInvObject(id, newValue);
			break;
		}
	}
	
	private void updateNode(TreeElement element, String newValue){
		((Nameable)element.getObject()).setName(newValue);
	}
}