package de.ba_leipzig.discoinv.tree;

import java.awt.*;
import java.sql.SQLException;

import javax.swing.event.*;
import javax.swing.tree.*;

import de.ba_leipzig.discoinv.ExceptionHandler;
import de.ba_leipzig.discoinv.db.DBManagerLoadException;
import de.ba_leipzig.discoinv.db.DBManagerFactory;
import de.ba_leipzig.discoinv.db.DatabaseManager;

public class TreeLoadHandler implements TreeWillExpandListener {
	private boolean placesLoaded;
	private boolean beveragesLoaded;
	private boolean miscLoaded;
	private int dbManagerID;
	private Component ui;
	private TreeManager treeManager;
	
	public TreeLoadHandler(Component ui, TreeManager treeManager){
		if(ui == null || treeManager == null){
			throw new IllegalArgumentException("parameters must not be null!");
		}
		
		this.ui = ui;
		this.treeManager = treeManager;
	}
	
	public void treeWillCollapse(TreeExpansionEvent event)
	throws ExpandVetoException {
		// nothing to do
	}
	
	public void treeWillExpand(TreeExpansionEvent event)
	throws ExpandVetoException {
		Cursor cursor = ui.getCursor();
		
		try {
			DatabaseManager dbManager = DBManagerFactory.getDatabaseManager();
			
			if(	placesLoaded && beveragesLoaded && miscLoaded &&
				dbManagerID == dbManager.getDBManagerID()){
				return;
			}
			
			placesLoaded = beveragesLoaded = miscLoaded = false;
			dbManagerID = dbManager.getDBManagerID();
			
			ui.setCursor(new Cursor(Cursor.WAIT_CURSOR));
			
			TreePath path = event.getPath();
			
			if(path.equals(treeManager.getPlacesPath())){
				treeManager.loadPlaces(dbManager);
				placesLoaded = true;
			}
			else if(path.equals(treeManager.getBeveragesPath())){
				treeManager.loadBeverages(dbManager);
				beveragesLoaded = true;
			}
			else if(path.equals(treeManager.getMiscPath())){
				treeManager.loadMisc(dbManager);
				miscLoaded = true;
			}
		}
		catch(DBManagerLoadException e){
			new ExceptionHandler(ui).handle(e);
			throw new ExpandVetoException(event);
		}
		catch(SQLException e){
			String msg = "Beim ffnen des Zweigs ist ein Datenbankfehler aufgetreten.";
			new ExceptionHandler(ui).handle(e, msg);
			throw new ExpandVetoException(event);
		}
		finally {
			ui.setCursor(cursor);
		}
	}
}