package de.ba_leipzig.discoinv.tree;

import java.sql.SQLException;
import java.util.*;

import javax.swing.*;
import javax.swing.tree.*;

import de.ba_leipzig.discoinv.db.*;

public class TreeManager {
	private static final int PLACES_INDEX = 0;
	private static final int BEVERAGES_INDEX = 1;
	private static final int MISC_INDEX = 2;
	
	private JTree objectTree;
	
	public void editNode(TreeNode node){
		objectTree.startEditingAtPath(getPathForNode(node));
	}
	
	public TreeNode getBeveragesNode(){
		return (TreeNode)getRootNode().getChildAt(BEVERAGES_INDEX);
	}
	
	public TreePath getBeveragesPath(){
		return getPathForNode(getBeveragesNode());
	}
	
	public TreeNode getMiscNode(){
		return (TreeNode)getRootNode().getChildAt(MISC_INDEX);
	}
	
	public TreePath getMiscPath(){
		return getPathForNode(getMiscNode());
	}
	
	public TreePath getPathForNode(TreeNode node){
		return new TreePath(node.getPath());
	}
	
	public TreeNode getPlacesNode(){
		return (TreeNode)getRootNode().getChildAt(PLACES_INDEX);
	}
	
	public TreePath getPlacesPath(){
		return getPathForNode(getPlacesNode());
	}
	
	public TreeNode getRootNode(){
		return getTreeModel().getRoot();
	}
	
	public TreePath getRootPath(){
		return getPathForNode(getRootNode());
	}
	
	public TreePath getSelectedPath(){
		return objectTree.getSelectionPath();
	}
	
	public boolean isDeleteableNodePath(TreePath path){
		return 	!path.equals(getRootPath()) 		&& 
				!path.equals(getPlacesPath())		&&
				!path.equals(getBeveragesPath())	&&
				!path.equals(getMiscPath());
	}
	
	public boolean isRootSelected(){
		TreePath selectedPath = getSelectedPath();
		
		if(selectedPath != null && selectedPath.equals(getRootPath())){
			return true;
		}
		else {
			return false;
		}
	}
	
	public void loadBeverages(DatabaseManager dbManager)
	throws SQLException {
		getTreeModel().unloadBeverages();
		Set<Beverage> beverages = dbManager.getBeverages();
		
		for(Beverage beverage : beverages){
			TreeElement<Beverage> element =
				new TreeElement<Beverage>(beverage, TreeObjectType.beverage);
			TreeNode node =	new TreeNode(element, false);
			getTreeModel().addChildToBeverages(node);
		}
	}
	
	public void loadMisc(DatabaseManager dbManager)
	throws SQLException {
		getTreeModel().unloadMisc();
		Set<InvObject> misc = dbManager.getMisc();
		
		for(InvObject object : misc){
			TreeElement<InvObject> element =
				new TreeElement<InvObject>(object, TreeObjectType.misc);
			TreeNode node =	new TreeNode(element, false);
			getTreeModel().addChildToMisc(node);
		}
	}
	
	public void loadPlaces(DatabaseManager dbManager)
	throws SQLException {
		getTreeModel().unloadPlaces();
		Set<Place> places = dbManager.getPlaces();
		
		for(Place place : places){
			TreeElement<Place> element =
				new TreeElement<Place>(place, TreeObjectType.place);
			TreeNode node = new TreeNode(element, false);
			getTreeModel().addChildToPlaces(node);
		}
	}
	
	public void nodeChanged(TreeNode node){
		getTreeModel().nodeChanged(node);
	}
	
	public void selectBeverages(boolean expand){
		selectNode(getBeveragesPath(), expand);
	}
	
	public void selectNode(TreeNode node){
		objectTree.expandPath(getPathForNode(node));
	}
	
	public void selectMisc(boolean expand){
		selectNode(getMiscPath(), expand);
	}
	
	public void selectPlaces(boolean expand){
		selectNode(getPlacesPath(), expand);
	}
	
	public void selectRoot(){
		selectNode(getRootPath(), true);
	}
	
	public void setObjectTree(JTree objectTree){
		if(objectTree == null){
			throw new IllegalArgumentException("objectTree must not be null!");
		}
		
		this.objectTree = objectTree;
	}
	
	private TreeModel getTreeModel(){
		return (TreeModel)objectTree.getModel();
	}
	
	private void selectNode(TreePath path, boolean expand){
		objectTree.expandPath(getRootPath());
		
		// Schliesse alle direkten Kinder von root
		for(Enumeration children = getRootNode().children();
			children.hasMoreElements();)
		{
			Object[] childPath = ((TreeNode)children.nextElement()).getPath();
			objectTree.collapsePath(new TreePath(childPath));
		}
		
		objectTree.setSelectionPath(path);
		if(expand && !path.equals(getRootPath())) objectTree.expandPath(path);
	}
}