package de.matthiasrothe.util.measuring;

import java.util.*;

/**
 * This class represents the physical notion of a gram as a measuring unit.
 *  
 * @author Matthias Rothe
 * @since 27.08.2007
 */
public final class Gram extends AbstractMeasuringUnit implements MeasuringUnit {

	/**
	 * Public due to requirements of {@link ResourceBundle}. Do not instantiate
	 * yourself!
	 * 
	 * @author Matthias
	 */
	public static class Name extends ListResourceBundle {
		private Object[][] contents = new Object[][]{
				{"abbreviation", "g"},
				{"fullname", "Gram"}};
		
		protected Object[][] getContents(){
			return contents;
		}
	}
	
	/**
	 * Public due to requirements of {@link ResourceBundle}. Do not instantiate
	 * yourself!
	 * 
	 * @author Matthias
	 */
	public static class Name_de extends ListResourceBundle {
		private Object[][] contents = new Object[][]{
				{"abbreviation", "g"},
				{"fullname", "Gramm"}};
		
		protected Object[][] getContents() {
			return contents;
		}
	}
	
	private static final long serialVersionUID = -802851760199678040L;
	
	private static transient final String localeBaseName =
		"de.matthiasrothe.util.measuring.Gram$Name";
	
	private transient HashSet<Locale> locales = new HashSet<Locale>();
	
	/**
	 * Constructs a new <i>Gram</i> object and sets the prefix to
	 * {@link Prefix#none}.
	 */
	public Gram(){
		initLocales();
	}
	
	/**
	 * Constructs a new <i>Gram</i> object and sets the prefix to the prefix
	 * provided.
	 * 
	 * @param prefix The {@link Prefix} to be used with the new instance
	 */
	public Gram(Prefix prefix){
		super(prefix);
		initLocales();
	}
	
	private void initLocales(){
		locales.add(Locale.ENGLISH);
		locales.add(Locale.GERMAN);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public Set<Locale> getAvailableLocales(){
		return locales;
	}

	protected String getLocaleBaseName() {
		return localeBaseName;
	}
}