package de.matthiasrothe.util.measuring;

import java.util.*;

/**
 * This class represents the physical notion of a liter as a measuring unit.
 *  
 * @author Matthias Rothe
 * @since 27.08.2007
 */
public final class Liter
	extends AbstractMeasuringUnit
		implements MeasuringUnit {

	/**
	 * Public due to requirements of {@link ResourceBundle}. Do not instantiate
	 * yourself!
	 * 
	 * @author Matthias
	 */
	public static class Name extends ListResourceBundle {
		private Object[][] contents = new Object[][]{
				{"abbreviation", "l"},
				{"fullname", "liter"}};
		
		protected Object[][] getContents(){
			return contents;
		}
	}
	
	/**
	 * Public due to requirements of {@link ResourceBundle}. Do not instantiate
	 * yourself!
	 * 
	 * @author Matthias
	 */
	public static class Name_en_GB extends ListResourceBundle {
		private Object[][] contents = new Object[][]{
				{"abbreviation", "l"},
				{"fullname", "litre"}};
		
		protected Object[][] getContents(){
			return contents;
		}
	}
	
	/**
	 * Public due to requirements of {@link ResourceBundle}. Do not instantiate
	 * yourself!
	 * 
	 * @author Matthias
	 */
	public static class Name_de extends ListResourceBundle {
		private Object[][] contents = new Object[][]{
				{"abbreviation", "l"},
				{"fullname", "Liter"}};
		
		protected Object[][] getContents() {
			return contents;
		}
	}
	
	private static final long serialVersionUID = -7646065326671585841L;
	
	private static transient final String localeBaseName =
		"de.matthiasrothe.util.measuring.Liter$Name";
	
	private transient HashSet<Locale> locales = new HashSet<Locale>();
	
	/**
	 * Constructs a new <i>Gram</i> object and sets the prefix to
	 * {@link Prefix#none}.
	 */
	public Liter(){
		initLocales();
	}
	
	/**
	 * Constructs a new <i>Liter</i> object and sets the prefix to the prefix
	 * provided.
	 * 
	 * @param prefix The {@link Prefix} to be used with the new instance
	 */
	public Liter(Prefix prefix){
		super(prefix);
		initLocales();
	}
	
	private void initLocales(){
		locales.add(Locale.ENGLISH);
		locales.add(Locale.GERMAN);
		locales.add(new Locale("en", "GB"));
	}
	
	/**
	 * {@inheritDoc}
	 */
	public Set<Locale> getAvailableLocales() {
		return locales;
	}
	
	/**
	 * {@inheritDoc}
	 */
	protected String getLocaleBaseName() {
		return localeBaseName;
	}
}