package de.matthiasrothe.util.measuring;

import java.io.Serializable;
import java.util.*;

public interface MeasuringUnit extends Serializable, Comparable<MeasuringUnit> {
	public enum Prefix {
		yocto {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return -24
			 */
			@Override public int getExponent(){
				return -24;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E-24
			 */
			@Override public double getMultiplicator(){
				return 1.0E-24;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return y
			 */
			@Override public String getSymbol(){
				return "y";
			}
		},
		
		zepto {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return -21
			 */
			@Override public int getExponent(){
				return -21;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E-21
			 */
			@Override public double getMultiplicator(){
				System.out.println("zepto.getMultiplikator()");
				return 1.0E-21;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return z
			 */
			@Override public String getSymbol(){
				System.out.println("zepto.getSymbol()");
				return "z";
			}
		},
		
		atto {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return -18
			 */
			@Override public int getExponent(){
				return -18;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E-18
			 */
			@Override public double getMultiplicator(){
				return 1.0E-18;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return a
			 */
			@Override public String getSymbol(){
				return "a";
			}
		},
		
		femto {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return -15
			 */
			@Override public int getExponent(){
				return -15;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E-15
			 */
			@Override public double getMultiplicator(){
				return 1.0E-15;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return f
			 */
			@Override public String getSymbol(){
				return "f";
			}
		},
		
		pico {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return -12
			 */
			@Override public int getExponent(){
				return -12;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E-12
			 */
			@Override public double getMultiplicator(){
				return 1.0E-12;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return p
			 */
			@Override public String getSymbol(){
				return "p";
			}
		},
		
		nano {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return -9
			 */
			@Override public int getExponent(){
				return -9;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E-9
			 */
			@Override public double getMultiplicator(){
				return 1.0E-9;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return n
			 */
			@Override public String getSymbol(){
				return "n";
			}
		},
		
		micro {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return -6
			 */
			@Override public int getExponent(){
				return -6;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E-6
			 */
			@Override public double getMultiplicator(){
				return 1.0E-6;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return 
			 */
			@Override public String getSymbol(){
				return "";
			}
		},
		
		milli {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return -3
			 */
			@Override public int getExponent(){
				return -3;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E-3
			 */
			@Override public double getMultiplicator(){
				return 1.0E-3;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return m
			 */
			@Override public String getSymbol(){
				return "m";
			}
		},
		
		centi {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return -2
			 */
			@Override public int getExponent(){
				return -2;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E-2
			 */
			@Override public double getMultiplicator(){
				return 1.0E-2;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return c
			 */
			@Override public String getSymbol(){
				return "c";
			}
		},
		
		deci {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return -1
			 */
			@Override public int getExponent(){
				return -1;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E-1
			 */
			@Override public double getMultiplicator(){
				return 1.0E-1;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return d
			 */
			@Override public String getSymbol(){
				return "d";
			}
		},
		
		none,
		
		deca {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return 1
			 */
			@Override public int getExponent(){
				return 1;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E1
			 */
			@Override public double getMultiplicator(){
				return 1.0E1;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return da
			 */
			@Override public String getSymbol(){
				return "da";
			}
		},
		
		hecto {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return 2
			 */
			@Override public int getExponent(){
				return 2;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E2
			 */
			@Override public double getMultiplicator(){
				return 1.0E2;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return h
			 */
			@Override public String getSymbol(){
				return "h";
			}
		},
		
		kilo {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return 3
			 */
			@Override public int getExponent(){
				return 3;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E3
			 */
			@Override public double getMultiplicator(){
				return 1.0E3;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return k
			 */
			@Override public String getSymbol(){
				return "k";
			}
		},
		
		mega {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return 6
			 */
			@Override public int getExponent(){
				return 6;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E6
			 */
			@Override public double getMultiplicator(){
				return 1.0E6;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return M
			 */
			@Override public String getSymbol(){
				return "M";
			}
		},
		
		giga {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return 9
			 */
			@Override public int getExponent(){
				return 9;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E9
			 */
			@Override public double getMultiplicator(){
				return 1.0E9;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return G
			 */
			@Override public String getSymbol(){
				return "G";
			}
		},
		
		tera {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return 12
			 */
			@Override public int getExponent(){
				return 12;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E12
			 */
			@Override public double getMultiplicator(){
				return 1.0E12;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return T
			 */
			@Override public String getSymbol(){
				return "T";
			}
		},
		
		peta {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return 15
			 */
			@Override public int getExponent(){
				return 15;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E15
			 */
			@Override public double getMultiplicator(){
				return 1.0E15;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return P
			 */
			@Override public String getSymbol(){
				return "P";
			}
		},
		
		exa {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return 18
			 */
			@Override public int getExponent(){
				return 18;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E18
			 */
			@Override public double getMultiplicator(){
				return 1.0E18;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return E
			 */
			@Override public String getSymbol(){
				return "E";
			}
		},
		
		zetta {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return 21
			 */
			@Override public int getExponent(){
				return 21;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E21
			 */
			@Override public double getMultiplicator(){
				return 1.0E21;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return Z
			 */
			@Override public String getSymbol(){
				return "Z";
			}
		},
		
		yotta {
			/**
			 * Returns the base 10 exponent for the prefix.<br>
			 * @return 24
			 */
			@Override public int getExponent(){
				return 24;
			}
			
			/**
			 * Returns the base 10 multiplicator for the prefix.<br>
			 * @return 1.0E24
			 */
			@Override public double getMultiplicator(){
				return 1.0E24;
			}
			
			/**
			 * Returns the symbol for the prefix.<br>
			 * @return Y
			 */
			@Override public String getSymbol(){
				return "Y";
			}
		};
		
		/**
		 * Returns the base 10 exponent for the prefix.<br>
		 * E.g. {@link MeasuringUnit.Prefix#kilo#getExponent()} returns 3.
		 * 
		 * @return the base 10 exponent for the prefix 
		 */
		public int getExponent(){
			return 0;
		}
		
		/**
		 * Returns the base 10 multiplicator for the prefix.<br>
		 * E.g. {@link MeasuringUnit.Prefix#kilo#getMultiplicator()} returns 1000.
		 * 
		 * @return the multiplicator for the prefix
		 */
		public double getMultiplicator(){
			return 1.0E0;
		}
		
		/**
		 * Returns the symbol for the prefix.<br>
		 * E.g. {@link MeasuringUnit.Prefix#kilo#getSymbol()} returns
		 * <code>kilo</code>.
		 * 
		 * @return the symbol for the prefix
		 */
		public String getSymbol(){
			return "";
		}
	}
	
	/**
	 * Compares two <i>MeasuringUnits</i> of the same type based on their
	 * prefixes and therefore serves as a shorthand for calling
	 * <p>
	 * <code>
	 * getPrefix().compareTo(unit.getPrefix())
	 * </code>
	 * <p>
	 * on this object provided that unit is of the same type as this object.
	 * 
	 * @param unit The unit this object is to be compared with
	 * @return the same as calling getPrefix().compareTo(unit.getPrefix()) on
	 * this object provided that unit is of the same type as this object
	 * @throws ClassCastException if unit is not of the same type as this
	 * object
	 */
	public int compareTo(MeasuringUnit unit);
	
	/**
	 * Returns the abbreviation of the unit with the prefix, if any is set,
	 * according to the English locale.
	 * 
	 * @return the abbreviation
	 */
	public String getAbbreviation();
	
	/**
	 * Returns the abbreviation of the unit with the prefix, if any is set,
	 * according to the given Locale.
	 * 
	 * @return the abbreviation
	 */
	public String getAbbreviation(Locale locale);
	
	/**
	 * Returns a {@link Set} of all {@link Locale}s for which the methods
	 * {@link #getAbbreviation(Locale)} and {@link #getFullName(Locale)}
	 * can return localized {@link String}s.
	 * 
	 * @return a Set of all Locales available
	 */
	public Set<Locale> getAvailableLocales();
	
	/**
	 * Returns the full name of the unit with the, prefix if any is set,
	 * according to the English locale.
	 * 
	 * @return the full name
	 * @see Locale#ENGLISH
	 */ 
	public String getFullName();
	
	/**
	 * Returns the full name of the unit with the prefix, if any is set,
	 * according to the given {@link Locale}.
	 * 
	 * @return the full name
	 * @see Locale
	 */
	public String getFullName(Locale locale);
	
	/**
	 * @return the prefix
	 */
	public Prefix getPrefix();
	
	/**
	 * Sets the prefix
	 */
	public void setPrefix(Prefix prefix);
}